/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package mathutil;

/**
 * <p>Tda slou k uchovn jednoho komplexnho sla.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class Complex {
  private double real;
  private double im;

  public Complex() {}

  public Complex(double real) {
    this.real = real;
  }

  public Complex(double real, double im) {
    this.real = real;
    this.im = im;
  }

  public Complex(Complex c) {
    this.real = c.real;
    this.im = c.im;
  }

  public double getReal() {
    return real;
  }

  public double getIm() {
    return im;
  }

  /**
   * Dl komplexn slo relnm slem
   * @param num double
   */
  public void divide(double num) {
    real /= num;
    im /= num;
  }

  /**
   * St dv komplexn sla. Pvodn komplexn slo zstv beze zmny.
   * @param c Complex druh stanec
   * @return Complex souet
   */
  public Complex add(Complex c) {
    return new Complex(real + c.real, im + c.im);
  }

  /**
   * Odet dv komplexn sla. Pvodn komplexn slo zstv beze zmny.
   * @param c Complex odetan slo
   * @return Complex vsledek
   */
  public Complex substract(Complex c) {
    return new Complex(real - c.real, im - c.im);
  }

  /**
   * Nsob dv komplexn sla. Pvodn komplexn slo zstv beze zmny.
   * @param c Complex
   * @return Complex nsobek
   */
  public Complex multiple(Complex c) {
    double r = real * c.real - im * c.im;
    double i = real * c.im + im * c.real;
    return new Complex(r, i);
  }

  public boolean equals(Object o) {
    if(o instanceof Complex) {
      Complex c = (Complex) o;
      return (real == c.real) && (im == c.im);
    }
    return false;
  }

  public String toString() {
    return real + " " + im +"i";
  }
}
