/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package mathutil;

/**
 * <p>Tda implementuje vpoet spektrogramu.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class Spectrogram {
  private Spectrogram() {}

  /**
   * Vypot spektrogram pro zadan data
   * @param data ComplexVector vstupn data
   * @param frequency int maximln frekvence spektrogramu
   * @param fftLength int dlka FFT
   * @param window int velikost okna FFT
   * @param overlap int o kolik se okno v jednom kroku posune
   * @return Matrix
   */
  public static Matrix specgram(ComplexVector data, int frequency, int fftLength, int window, int overlap) {
    Matrix res = new Matrix((int) Math.ceil(data.length()/(double)overlap), frequency);
    int col = 0;
    for(int i = 0; i < data.length(); i += overlap) {
      ComplexVector cv = FFT.fft(data.subVector(i, i+window), fftLength);
      Vektor power = cv.getPowerSpectrum();
      for(int j = 0; j < frequency; j++) {
        res.set(col, j, power.get(j));
      }
      col++;
    }
    return res;
  }
}
