/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package mathutil;

/**
 * <p>Tda uchovv vektor (adu) relnch sel.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class Vektor {
  private int length;
  private double[] vector;

  public Vektor() {
    this(0);
  }

  /**
   * Inicializuje vektro na danou dlku
   * @param length int
   */
  public Vektor(int length) {
    this.length = length;
    vector = new double[length];
  }

  public Vektor(Vektor vector) {
    this.length = vector.length;
    this.vector = new double[length];
    for (int i = 0; i < length; i++) {
      this.vector[i] = vector.vector[i];
    }
  }

  public Vektor(double[] vector) {
    this.length = vector.length;
    this.vector = new double[length];
    for (int i = length-1; i >=0; i--) {
      this.vector[i] = vector[i];
    }
  }

  public int getLength() {
    return length;
  }

  /**
   * vypot skalrn souin dvou vektor
   * @param vector Vektor
   * @return double
   */
  public double scalarProduct(Vektor vector) {
    if (length != vector.length) {
      throw new Error("Delky vektoru musi souhlasit");
    }

    double res = 0;
    for (int i = 0; i < length; i++) {
      res += this.vector[i] * vector.vector[i];
    }
    return res;
  }

  /**
   * vynsob vektor relnm slem
   * @param coef double
   */
  public void multipleBy(double coef) {
    for (int i = 0; i < length; i++) {
      this.vector[i] *= coef;
    }
  }

  /**
   * vypot podl vektoru prvek po prvku
   * @param vector Vektor
   * @return double
   */
  public double divideBy(Vektor vector) {
    if (length != vector.length) {
      throw new Error("Delky vektoru musi souhlasit");
    }

    double res = 0;
    for (int i = 0; i < length; i++) {
      res += this.vector[i] / vector.vector[i];
    }
    return res;
  }

  /**
   * vydl vektor relnm slem
   * @param coef double
   */
  public void divideBy(double coef) {
    for (int i = 0; i < length; i++) {
      this.vector[i] /= coef;
    }
  }

  /**
   * Vrac souet prvek po prvku dvou vektor
   * @param vector Vektor
   * @return Vektor
   */
  public Vektor add(Vektor vector) {
    if (length != vector.length) {
      throw new Error("Delky vektoru musi souhlasit");
    }

    Vektor res = new Vektor(length);
    for (int i = 0; i < length; i++) {
      res.vector[i] = this.vector[i] + vector.vector[i];
    }
    return res;
  }

  /**
   * pite ke kadmu prvku vektoru dan slo
   * @param coef double
   */
  public void add(double coef) {
    for (int i = 0; i < length; i++) {
      this.vector[i] += coef;
    }
  }

  /**
   * vechny prvky vektoru pevede na opan sla
   * @return Vektor
   */
  public Vektor negated() {
    Vektor res = new Vektor(length);
    for (int i = 0; i < length; i++) {
      res.vector[i] = -vector[i];
    }
    return res;
  }

  /**
   * Vrac rozdl prvek po prvku dvou vektor
   * @param vector Vektor
   * @return Vektor
   */
  public Vektor substract(Vektor vector) {
    if (length != vector.length) {
      throw new Error("Delky vektoru musi souhlasit");
    }

    Vektor res = new Vektor(length);
    for (int i = 0; i < length; i++) {
      res.vector[i] = this.vector[i] - vector.vector[i];
    }
    return res;
  }

  /**
   * odete od kadho prvku vektoru dan slo
   * @param coef double
   */
  public void substract(double coef) {
    for (int i = 0; i < length; i++) {
      this.vector[i] -= coef;
    }
  }

  public double get(int i) {
    return vector[i];
  }

  public void set(int i, double value) {
    vector[i] = value;
  }

  public double[] toArray() {
    return vector.clone();
  }

  public String toString() {
    StringBuffer buf = new StringBuffer(length * 3);
    buf.append(vector[0]);
    for (int i = 1; i < length; i++) {
      buf.append(", ");
      buf.append(vector[i]);
    }
    buf.append("\n");
    return buf.toString();
  }

  public boolean equals(Object o) {
    if (o instanceof Vektor) {
      Vektor v = (Vektor) o;
      if (this.length != v.length) {
        return false;
      }
      for (int i = 0; i < length; i++) {
        if (vector[i] != v.vector[i]) {
          return false;
        }
      }
      return true;
    }
    return false;
  }

  /**
   * zvt nebo zmen vektor na novou velikost
   * @param length int
   */
  public void resize(int length) {
    double[] newVector = new double[length];
    int nl = Math.min(this.length, length);
    for (int i = 0; i < nl; i++) {
      newVector[i] = vector[i];
    }
    vector = newVector;
    this.length = length;
  }
}
