/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data;

import java.awt.event.*;
import javax.swing.JComboBox;
import spirals.motol.database.Data;
import spirals.data.viewers.TabletDataViewer;

/**
 * <p>ComboBox s daty pro zobrazen.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class DataComboBox
    extends JComboBox {
  private TabletDataViewer tdv;
  private Data data;
  private boolean all;
  private static int selected = 0;
  private static String[] list = {
      "hel", "x", "y", "rychlost", "vx", "vy", "tlak"};

  public DataComboBox(TabletDataViewer tdv, Data data, boolean all) {
    super(list);
    this.data = data;
    this.all = all;
    this.tdv = tdv;
    this.setSelectedIndex(selected);
    this.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        repaintAction();
      }
    });
  }

  private void repaintAction() {
    this.selected = this.getSelectedIndex();
    tdv.updateData();
    tdv.repaint();
  }

  public double[] getData() {
    TabletData td = null;
    if(all) td = data.getAllData();
    else td = data.getData();

    switch(selected) {
      case 1: return td.getDoubleX();
      case 2: return td.getDoubleY();
      case 3: return td.getVelocity();
      case 4: return td.getVelocityX();
      case 5: return td.getVelocityY();
      case 6:
        int size = td.size();
        double[] pressure = new double[size];

        for (int i = 0; i < size; i++) {
          pressure[i] = td.get(i).getNormalPressure();
        }
        return pressure;
      default: return td.getAngle();
    }
  }

  public double getKoeficient() {
    switch(selected) {
      case 1:
      case 2: return .000001;
      case 3: return .03;
      case 4:
      case 5: return .01;
      case 6: return .0002;
      default: return 10;
    }
  }
}
