/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data;

import javax.swing.*;
import spirals.data.*;
import spirals.data.viewers.*;
import spirals.data.doubleViewers.*;
import spirals.motol.database.Data;

/**
 * <p>Tda roziuje JComboBox. Jsou v n uloeny implemetovan prohlee dat.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class ViewersComboBox
    extends JComboBox {

  static private final String viewers[] = {
      "ZoomTabletDataViewer",
      "PressureTabletDataViewer",
      "LinearRegressionWidthTabletDataViever",
      "FiltWidthTabletDataViever",
      "TimeTabletDataViewer",
      "TimeDifTabletDataViewer",
      "LengthTabletDataViewer",
      "FFTTabletDataViewer",
      "SpecgramTabletDataViewer"
  };

  static private final String names[] = {
      "Zobrazen s lupou",
      "Zobrazen tlaku",
      "Vzdlenost od pmky",
      "Vzdlenost od filtrovanho signlu",
      "Zvislost na ase",
      "Zvislost rozdlu na ase",
      "Zvislost na dlce",
      "FFT",
      "Spektrogram"
  };

  public ViewersComboBox() {
    super(names);
  }

  /**
   * Vytvo vybran prohle s danmi parametry
   * @param screen JPanel
   * @param controls JPanel
   * @param data Data
   * @param showUp boolean
   * @return TabletDataViewer
   */
  public TabletDataViewer invokeClass(JPanel screen, JPanel controls, Data data,
                                      boolean all, boolean showUp) {
    switch (this.getSelectedIndex()) {
      case 1:
        return new PressureTabletDataViewer(screen, controls, data, all,
                                                 showUp);
      case 2:
        return new LinearRegressionWidthTabletDataViewer(screen, controls, data,
            all, showUp);
      case 3:
        return new FiltWidthTabletDataViewer(screen, controls, data, all,
                                             showUp);
      case 4:
        return new TimeTabletDataViewer(screen, controls, data, all,
                                             showUp);
      case 5:
        return new TimeDifTabletDataViewer(screen, controls, data, all,
                                                showUp);
      case 6:
        return new LengthTabletDataViewer(screen, controls, data, all,
                                               showUp);
      case 7:
        return new FFTTabletDataViewer(screen, controls, data, all, showUp);
      case 8:
        return new SpecgramTabletDataViewer(screen, controls, data, all, showUp);
      default:
        return new ZoomTabletDataViewer(screen, controls, data, all, showUp);
    }
  }

  private String getClassName() {
    int index = this.getSelectedIndex();
    if ( (index >= 0) && (index < viewers.length)) {
      return "spirals.data." + viewers[index];
    }
    return null;
  }
}
