/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.doubleViewers;

import java.awt.*;
import javax.swing.*;

/**
 * <p>Zkladn tda pro zobrazovn dat ve spodnm panelu. Je st dvouokenho prohlee</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @see spirals.data.doubleViewers.DoubleViewer DoubleViewer
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class DoubleViewerBottom {
  /**
   * velikost okraje
   */
  private final int offset = 3;
  /**
   * index zatku zvraznovanch dat
   */
  protected int begin = -1;
  /**
   * index konce zvraznovanch dat
   */
  protected int end = -1;
  /**
   * x-ov pozice kurzuru
   */
  protected int whiteX = -1;
  protected double max = 0;
  protected double min = 0;
  protected double[] data;
  protected int size = 0;
  protected JPanel screen;

  public DoubleViewerBottom(JPanel screen, double[] data) {
    this.data = data;
    this.screen = screen;
    this.size = data.length;
  }

  public void paint(Graphics g) {
    int scrWidth = screen.getWidth();
    int scrHeight = screen.getHeight();

    // pomr vky pltna k rozsahu dat
    double koef = (scrHeight - 2 * offset) / (max - min);
    // pozice nejniho bodu
    int offsetFi = scrHeight + (int) (min * koef) - offset;

    g.setColor(Color.WHITE);
    g.fillRect(0, 0, scrWidth, scrHeight);

    g.setColor(Color.LIGHT_GRAY);
    g.drawLine(0, offsetFi, scrWidth, offsetFi);
    g.drawString(String.format("%.1f", max), 5, 15);
    g.drawString(String.format("%.1f", min), 5, scrHeight - 5);

    this.paintGraph(g, scrWidth - 2 * offset, offsetFi, koef);

    g.setColor(Color.LIGHT_GRAY);
    g.drawLine(whiteX, 0, whiteX, scrHeight);
  }

  /**
   * vykreslen vlastnch dat
   * @param g Graphics
   * @param scrWidth int
   * @param offsetFi int
   * @param koef double
   */
  protected void paintGraph(Graphics g, int scrWidth, int offsetFi, double koef) {
    int x, ox = 0;
    int y, oy = offsetFi;
    double xk = (double) size / (double) scrWidth;
    g.setColor(Color.BLACK);
    for (int i = 0; i < size; i++) {
      x = offset + (int) ( (double) i / xk);
      y = offsetFi - (int) (data[i] * koef);

      this.setDrawColor(g, i);
      g.drawLine(x, y, ox, oy);
      ox = x;
      oy = y;
    }
  }

  /**
   * vyznaen kus dat kresli erven
   * @param g Graphics
   * @param index int
   */
  protected void setDrawColor(Graphics g, int index) {
    if ( ( (index >= begin) && (index <= end)) ||
        ( (index >= end) && (index <= begin))) {
      g.setColor(Color.RED);
    }
    else {
      g.setColor(Color.BLACK);
    }
  }

  public void setBegin(int begin) {
    this.begin = begin;
    this.end = begin;
  }

  public void setEnd(int end) {
    this.end = end;
  }

  public void setWhiteX(int index) {
    this.whiteX = index;
  }

  public void setMin(double min) {
    this.min = min;
  }

  public void setMax(double max) {
    this.max = max;
  }

  public int getSize() {
    return size;
  }

  public int getOffset() {
    return offset;
  }
}
