/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.doubleViewers;

import java.awt.*;
import javax.swing.JPanel;

/**
 * <p>Doln panel - zobrazuje FFT.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class FFTBottom
    extends DoubleViewerBottom {
  /**
   * maximln zobrazovan frekcence
   */
  protected int maxFrequency;
  /**
   * poet vzork odpovdajc maximln frekvenci
   */
  protected int fftSize;

  /**
   * @param screen JPanel
   * @param data double[] hodnoty FFT
   */
  public FFTBottom(JPanel screen, double[] data) {
    super(screen, data);
  }

  public void paint(Graphics g) {
    int scrWidth = screen.getWidth();
    int scrHeight = screen.getHeight() - 3;

    double koef = scrHeight / max;

    g.setColor(Color.WHITE);
    g.fillRect(0, 0, scrWidth, screen.getHeight());

    g.setColor(Color.LIGHT_GRAY);
    for (int i = 0; i < maxFrequency; i += 10) {
      int x = i * scrWidth / maxFrequency;
      g.drawLine(x, 0, x, screen.getHeight());
    }

    g.drawString(String.format("%.1f", max), 5, 15);

    int x, ox = 0;
    int y, oy = 0;
    g.setColor(Color.BLACK);
    for (int i = 0; i < fftSize; i++) {
      x = (int) ( (double) i / (double) fftSize * (double) scrWidth);
      y = scrHeight - (int) (data[i] * koef);

      g.drawLine(x, y, ox, oy);
      ox = x;
      oy = y;
    }
    g.setColor(Color.LIGHT_GRAY);
  }

  public void setMaxFrequency(int frequency) {
    maxFrequency = frequency;
    fftSize = (int) (data.length / 1000.0 * maxFrequency);
  }
}
