/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.doubleViewers;

import javax.swing.*;
import spirals.motol.database.Data;
import mathutil.*;

/**
 * <p>Zobrazuje FFT.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class FFTTabletDataViewer
    extends TimeDifTabletDataViewer {
  /**
   * vka zobrazovanho grafu bude oznuta na thle hladin
   */
  protected static final double maxPower = 10000;
  /**
   * nejvy zobrazovan frekvence
   */
  protected static final int maxFrequency = 100;
  /**
   * velikost FFT
   */
  protected static final int fftSize = 65536;
  protected ComplexVector fft;

  public FFTTabletDataViewer(JPanel screen, JPanel controls, Data data,
                             boolean all, boolean showUp) {
    super(screen, controls, data, all, showUp);
  }

  protected void setBottom(double[] vect, double maxPower, int maxFrequency) {
    bottomViewer = new FFTBottom(bottom, vect);
    bottomViewer.setMax(maxPower / dcb.getKoeficient());
    ( (FFTBottom) bottomViewer).setMaxFrequency(maxFrequency);
  }

  public void updateData() {
    super.updateData();
    if(dcb != null) {
      fft = FFT.fft(new ComplexVector(dif), fftSize);
      this.setBottom(fft.getPowerSpectrum().toArray(), maxPower, maxFrequency);
    }
  }

  public String getDescription() {
    return "Graf FFT dlky " + fftSize +
        " potan s odstrannm stejnosmrn sloky.";
  }
}
