/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.doubleViewers;

import javax.swing.*;
import spirals.motol.database.Data;
import spirals.data.DataComboBox;

/**
 * <p>Prohle kter pouv cel data. Zobrazuje se pi vyznaen platnho seku dat po jejich sejmut z tabletu.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class TimeTabletDataViewer
    extends DoubleViewerDrag {
  protected DataComboBox dcb;

  protected double minFi, maxFi;
  protected double[] angle;

  public TimeTabletDataViewer(JPanel screen, JPanel controls, Data data,
                                   boolean all, boolean showUp) {
    super(screen, controls, data, all, showUp);
    this.install();
  }

  private void install() {
    dcb = new DataComboBox(this, data, all);
    controls.add(dcb);
    topViewer = new TADoubleViewerTop(top, topControls, data, all, showUp);
    this.updateData();
  }

  protected void setBottom(double[] vect, double min, double max) {
    bottomViewer = new DoubleViewerBottom(bottom, vect);
    bottomViewer.setMax(max);
    bottomViewer.setMin(min);
    if(all) {
      this.bottomViewer.setBegin(data.getStart());
      this.bottomViewer.setEnd(data.getEnd());
      this.topViewer.setBegin(data.getStart());
      this.topViewer.setEnd(data.getEnd());
    }
  }

  public void updateControls() {}

  /**
   * vybere vechna data
   */
  public void updateData() {
    super.updateData();

    if(dcb != null) {
      angle = dcb.getData();
      for(int i = 0; i < angle.length; i++) {
//        System.out.println(angle[i]);
      }
      minFi = Double.MAX_VALUE;
      maxFi = Double.MIN_VALUE;
      for (int i = angle.length - 1; i >= 0; i--) {
        minFi = Math.min(minFi, angle[i]);
        maxFi = Math.max(maxFi, angle[i]);
      }
      this.setBottom(angle, minFi, maxFi);
    }
  }

  public void disableControls() {}

  public JPanel getControls() {
    return ( (TADoubleViewerTop) topViewer).getControls();
  }

  public int getStart() {
    return bottomViewer.begin;
  }

  public int getEnd() {
    return bottomViewer.end;
  }

  /**
   * @return boolean vrac true, pokud byl vyznaen nov sek
   */
  public boolean changed() {
    return ( (TADoubleViewerTop) topViewer).changed();
  }

  public void setChanged(boolean changed) {
    ( (TADoubleViewerTop) topViewer).setChanged(changed);
  }

  public String getDescription() {
    return "Graf v zvislosti na ase. Taenm my pi stisknutm levm tlatku, lze v dolnm grafu zvraznit sek.";
  }
}
