/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.viewers;

import javax.swing.JPanel;
import spirals.motol.database.Data;
import java.awt.*;
import spirals.data.TabletData;
import javax.swing.event.ChangeListener;
import java.awt.event.KeyAdapter;
import javax.swing.event.ChangeEvent;
import java.awt.event.KeyEvent;
import javax.swing.SpinnerNumberModel;
import javax.swing.JSpinner;
import java.text.ParseException;

/**
 * <p>Tda roziuje prohle se zoomem.</p>
 * <p>Daty je proloena pmka zskan z linern regrese.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class LinearRegressionWidthTabletDataViewer
    extends ZoomTabletDataViewer {
  /**
   * vzdlenost mezi body pi pevzorkovn na funkci vzdlenosti
   */
  private final int resampleRate = 50;
  protected double a;
  protected double fi;
  protected double k;
  /**
   * polomr psu kolem pmky
   */
  private double offset = .2; // in cm
  /**
   * rozlien tabletu
   */
  private final double dpi = 1000;
  private JSpinner fieldOffset;

  public LinearRegressionWidthTabletDataViewer(JPanel screen, JPanel controls,
                                               Data data, boolean all,
                                               boolean showUp) {
    super(screen, controls, data, all, showUp);
    this.install();
  }

  private void install() {
    fieldOffset = new JSpinner(new SpinnerNumberModel(offset, .1, 2, .1));
    fieldOffset.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == e.VK_ENTER) {
          offsetChange();
        }
      }
    });
    fieldOffset.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        offsetChange();
      }
    });
    fieldOffset.setPreferredSize(new Dimension(50,
                                               (int) fieldOffset.
                                               getPreferredSize().
                                               getHeight()));
    controls.add(new Label("ka:"));
    controls.add(fieldOffset);
    controls.add(new Label("cm"));
  }

  /**
   * bastav nov rozpt
   */
  private void offsetChange() {
    try {
      fieldOffset.commitEdit();
    }
    catch (ParseException ex) {}
    offset = Double.parseDouble(fieldOffset.getValue().toString());
    screen.repaint();
  }

  public void paint(Graphics g) {
    this.clearWindow(g);
    this.drawTablet(g);

    this.paintLine(g);
    this.paintData(g, tabletData);
  }

  /**
   * kresl pmku a psy po stranch
   * @param g Graphics
   */
  protected void paintLine(Graphics g) {
    g.setColor(Color.ORANGE);
    this.paintSides(g, 0);
    g.setColor(Color.RED);
    this.paintSides(g, offset);
    this.paintSides(g, -offset);
  }

  /**
   * kresl prmku s posunutm
   * @param g Graphics
   * @param offset double posunut v centimetrech od pmky LR
   */
  private void paintSides(Graphics g, double offset) {
    if ( (a > -1) && (a < 1)) {
      int c = (int) ( (k + 2 * this.getOffsetY() + (offset * dpi) / Math.cos(fi)) *
                     this.getKoeficient());
      g.drawLine(this.getTabletOffsetX(),
                 (int) (screen.getHeight() - this.getTabletOffsetY() - c),
                 this.getTabletOffsetX() + this.getPictureWidth(),
                 screen.getHeight() - this.getTabletOffsetY()
                 - (int) (c + a * this.getPictureWidth()));
    }
    else {
      int c = (int) ( (k + (offset * dpi) / Math.cos(fi)) * this.getKoeficient());
      g.drawLine(this.getTabletOffsetX() +
                 (int) ( (this.getPictureHeight() - c) / a),
                 this.getTabletOffsetY(),
                 this.getTabletOffsetX() - (int) (c / a),
                 this.getTabletOffsetY() + this.getPictureHeight());
    }
  }

  /**
   * pokud je bod za hranic psu linern regrese, bud erven
   * @param g Graphics
   * @param index int
   */
  protected void setDrawColor(Graphics g, int index) {
    double x = tabletData.get(index).getX() + k / a;
    double y = x * Math.sin( -fi) +
        tabletData.get(index).getY() * Math.cos( -fi);
    if (Math.abs(y) > (offset * dpi)) {
      g.setColor(Color.RED);
    }
    else {
      g.setColor(Color.BLACK);
    }
  }

  /**
   * pot LR z pevzorkovanch dat
   */
  public void updateData() {
    super.updateData();

    TabletData td = tabletData.resampleEquidistant(resampleRate);
    this.a = td.getFi();
    this.fi = Math.atan(a);
    this.k = td.getOffset();
  }

  public String getDescription() {
    return "Daty je proloena pmka. Data se maj vejt do psu irokho 0.5 cm";
  }
}
