/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.viewers;

import spirals.motol.database.Data;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import tablet.TabletPoint;

/**
 * <p>Jednoduch zobrazova, kter um pibliovat / oddalovat a posouvat data.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class ZoomTabletDataViewer
    extends SimpleTabletDataViewer implements MouseListener,
    MouseMotionListener {

  protected int border = 6;

  /**
   * velikost piblen (mocnina dvou)
   */
  protected int detailLevel;
  private static final int MAXDETAIL = 128;
  /**
   * pozice stedu tabletu
   */
  protected int centerx;
  /**
   * pozice stedu tabletu
   */
  protected int centery;

  private JButton restore = new JButton(new AbstractAction("Vchoz") {
    public void actionPerformed(ActionEvent e) {
      detailLevel = 1;
      centerx = ti.getWidth() / 2;
      centery = ti.getHeight() / 2;
      updateControls();
      setScreenBorder();
      repaint();
    }
  });
  private JButton zoomIn = new JButton(new AbstractAction("Pibli") {
    public void actionPerformed(ActionEvent e) {
      detailLevel *= 2;
      updateControls();
      setScreenBorder();
      repaint();
    }
  });
  private JButton zoomOut = new JButton(new AbstractAction("Oddal") {
    public void actionPerformed(ActionEvent e) {
      detailLevel /= 2;
      updateControls();
      setScreenBorder();
      repaint();
    }
  });

  public ZoomTabletDataViewer(JPanel screen, JPanel controls, Data data,
                              boolean all, boolean showUp) {
    super(screen, controls, data, all, showUp);
    this.install();
  }

  private void install() {
    this.setScreenBorder();
    this.controls.add(zoomIn);
    this.controls.add(restore);
    this.controls.add(zoomOut);
    screen.addMouseMotionListener(this);
    screen.addMouseListener(this);
  }

  /**
   * vypot vchoz paramtery pro zobrazovn
   * @param screen JPanel
   */
  private void adjustPicture(JPanel screen) {
    this.centerx = ti.getWidth() -
        (tabletData.getMaxX() + tabletData.getMinX()) / 2;
    this.centery = ti.getHeight() -
        (tabletData.getMaxY() + tabletData.getMinY()) / 2;
    detailLevel = 2;
    if (tabletData.size() > 0) {
      // najdi nejvt piblen, kdy se data vejdou na obrazovku
      for (; detailLevel < MAXDETAIL; detailLevel *= 2) {
        double koef = this.getKoeficient();
        int scrHeight = screen.getHeight();
        int scrWidth = screen.getWidth();
        int height = scrHeight - this.getTabletOffsetY() -
            (int) (this.getOffsetY() * koef);
        int left = this.getTabletOffsetX() - (int) (this.getOffsetX() * koef);
        int offsetX = this.getOffsetX();
        int offsetY = this.getOffsetY();

        boolean end = false;
        for (int i = tabletData.size() - 1; i >= 0; i--) {
          TabletPoint point = tabletData.get(i);
          int x = (int) ( (point.getX() + offsetX) * koef) + left;
          int y = height - (int) ( (point.getY() + offsetY) * koef);
          if ( (x < 0) || (y < 0) || (x > scrWidth) || (y > scrHeight)) {
            end = true;
            break;
          }
        }
        if (end) {
          break;
        }
      }
    }
    detailLevel /= 2;
  }

  public void paint(Graphics g) {
    super.paint(g);
    this.drawTableOutline(g);
  }

  /**
   * vykreslen obrys tabletu
   * @param g Graphics
   */
  protected void drawTableOutline(Graphics g) {
    g.setColor(Color.DARK_GRAY);
    g.drawLine(this.getTabletOffsetX(), this.getTabletOffsetY(),
               this.getTabletOffsetX() + this.getPictureWidth(),
               this.getTabletOffsetY());
    g.drawLine(this.getTabletOffsetX(), this.getTabletOffsetY(),
               this.getTabletOffsetX(),
               this.getTabletOffsetY() + this.getPictureHeight());
    g.drawLine(this.getTabletOffsetX(),
               this.getTabletOffsetY() + this.getPictureHeight(),
               this.getTabletOffsetX() + this.getPictureWidth(),
               this.getTabletOffsetY() + this.getPictureHeight());
    g.drawLine(this.getTabletOffsetX() + this.getPictureWidth(),
               this.getTabletOffsetY(),
               this.getTabletOffsetX() + this.getPictureWidth(),
               this.getTabletOffsetY() + this.getPictureHeight());
  }

  /**
   * posunut v bodech tabletu
   * @return int
   */
  protected int getOffsetX() {
    return centerx - ti.getWidth() / 2;
  }

  /**
   * posunut v bodech tabletu
   * @return int
   */
  protected int getOffsetY() {
    return centery - ti.getHeight() / 2;
  }

  /**
   * pomr velikosti pltna k velikosti tabletu s ohledem na piblen
   * @return int
   */
  protected double getKoeficient() {
    return super.getKoeficient() * detailLevel;
  }

  /**
   * posunut v obrazovkovch bodech
   * @return int
   */
  protected int getTabletOffsetX() {
    return super.getTabletOffsetX() +
        (int) (this.getOffsetX() * this.getKoeficient());
  }

  /**
   * posunut v obrazovkovch bodech
   * @return int
   */
  protected int getTabletOffsetY() {
    return super.getTabletOffsetY() -
        (int) (this.getOffsetY() * this.getKoeficient());
  }

  public void updateControls() {
    super.updateControls();
    if (restore != null) {
      zoomOut.setEnabled(detailLevel > 1);
      restore.setEnabled(true);
      zoomIn.setEnabled(detailLevel < MAXDETAIL);
    }
  }

  public void disableControls() {
    zoomOut.setEnabled(false);
    restore.setEnabled(false);
    zoomIn.setEnabled(false);
  }

  public void updateData() {
    super.updateData();
    this.adjustPicture(screen);
  }

  /**
   * nakresl bravu okraje v zvislosti na piblen
   */
  private void setScreenBorder() {
    if (detailLevel == 1) {
      screen.setBorder(BorderFactory.createEtchedBorder(Color.BLUE, Color.CYAN));
    }
    else if (detailLevel >= MAXDETAIL) {
      screen.setBorder(BorderFactory.createEtchedBorder(Color.RED, Color.PINK));
    }
    else {
      screen.setBorder(BorderFactory.createEtchedBorder(Color.GREEN, Color.CYAN));
    }
  }

  public void mouseMoved(MouseEvent m) {}

  public void mouseExited(MouseEvent m) {}

  public void mouseEntered(MouseEvent m) {}

  public void mousePressed(MouseEvent m) {}

  public void mouseClicked(MouseEvent m) {}

  int oldx = -1;
  int oldy = -1;

  public void mouseReleased(MouseEvent m) {
    oldx = -1;
    oldy = -1;
    screen.setCursor(Cursor.getDefaultCursor());
  }

  /**
   * posune obraz tabletu pi taen myi
   * @param m MouseEvent
   */
  public void mouseDragged(MouseEvent m) {
    if (restore.isEnabled()) {
      screen.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
      if ( (oldx != -1) && (oldy != -1)) {
        double koef = this.getKoeficient();

        int relx = (int) ( (oldx - m.getX()) / koef);
        int rely = (int) ( (oldy - m.getY()) / koef);
        centerx -= relx;
        centery += rely;
        if (centerx < 0) {
          centerx = 0;
        }
        if (centery < 0) {
          centery = 0;
        }
        if (centerx > ti.getWidth()) {
          centerx = ti.getWidth();
        }
        if (centery > ti.getHeight()) {
          centery = ti.getHeight();
        }
        this.repaint();
      }
      oldx = m.getX();
      oldy = m.getY();
    }
  }

  public String getDescription() {
    return "Pi stisknut levho tlatka myi a taenm lze pohybovat obrzkem.\nPiblen/oddlen dat je se stedem lupy uprosted aktulnho vezu.";
  }
}
