/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.database;

import javax.swing.tree.*;
import javax.swing.event.*;

/**
 * <p>Interface between database and JTree.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class DatabaseModel implements TreeModel {
  /**
   * database
   */
  private Database database;
//  private List<TreeModelListener> listenerList = new ArrayList();

  /**
   * create database model from database
   */
  public DatabaseModel(Database database) {
    this.database = database;
  }

  /**
   * Add TreeModelListener. Need for the sake of Java
   */
  public void addTreeModelListener(TreeModelListener listener) {
/*    if ( listener != null && ! listenerList.contains( listener ) ) {
      listenerList.add(listener);
    }*/
  }

  /**
   * Remove TreeModelListener. Need for the sake of Java
   */
  public void removeTreeModelListener(TreeModelListener listener) {
/*    if ( listener != null ) {
      listenerList.remove( listener );
    }*/
  }

  /**
   * return root node
   */
  public Object getRoot() {
    return database;
  }

  /**
   * Test object whether it is leaf. Leaf is only visit
   */
  public boolean isLeaf(Object aNode) {
    if(aNode instanceof Database)
      return false;
    else if(aNode instanceof Pacient)
      return false;
    else if(aNode instanceof Visit)
      return true;
    System.err.println("Chyba - neplatny objekt ve strome");
    return true;
  }

  /**
   * Return number of childes of node. Visit has no child, Pacient has number of visits.
   * @param parent parent node
   */
  public int getChildCount(Object parent) {
    if(parent instanceof Database)
      return ((Database) parent).getTreeCount();
    else if(parent instanceof Pacient)
      return ((Pacient) parent).visitCount();
    else if(parent instanceof Visit)
      return 0;
    System.err.println("Chyba - neplatny objekt ve strome");
    return 0;
  }

  /**
   * return child of node with specified index
   * @param parent parent node
   * @param index index
   * @return Object
   */
  public Object getChild(Object parent, int index) {
    if(parent instanceof Database)
      return ((Database) parent).getTreePacient(index);
    else if(parent instanceof Pacient)
      return ((Pacient) parent).getVisit(index);
    else if(parent instanceof Visit)
      return null;
    System.err.println("Chyba - neplatny objekt ve strome");
    return null;
  }

  /**
   * return index of child node in parent node
   */
  public int getIndexOfChild(Object parent, Object child) {
    if(parent instanceof Database)
      return ((Database) parent).getTreeIndex((Pacient) child);
    else if(parent instanceof Pacient)
      return ((Pacient) parent).getIndex((Visit) child);
    else if(parent instanceof Visit)
      return 0;
    System.err.println("Chyba - neplatny objekt ve strome");
    return 0;
  }

  /**
   * Need for the sake of Java
   */
  public void valueForPathChanged(TreePath path, Object newValue) {
    System.err.println("Chyba - ve strome nepodporovano");
  }
}
