/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.database;

import spirals.motol.loaders.PacientLoader;
import java.util.*;
import java.lang.ref.SoftReference;

/**
 * <p>Class representing one pacient.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.0
 */
public class Pacient {
  /**
   * databze do kter pacient pat
   */
  private Database database = null;
  /**
   * tda pro nahrn pacienta
   */
  private PacientLoader loader = null;
  /**
   * ukazatel na seznam nvtv
   */
  private SoftReference visits = null;
  /**
   * ID = rodn slo
   */
  private String ID = "";
  private String firstname = "";
  private String surname = "";
  private Date born = null;
  /**
   * pravk?
   */
  private boolean rhp = true;

  public Pacient(Pacient p) {
    this.database = p.database;
    this.loader = p.loader;
    this.ID = p.ID;
    this.firstname = p.firstname;
    this.surname = p.surname;
    this.born = p.born;
    this.rhp = p.rhp;
  }

  public Pacient(Database database) {
    this(database, database.getLoader().getDefaultPacientLoader());
  }

  public Pacient(Database database, PacientLoader loader) {
    this.database = database;
    this.loader = loader;
    born = new Date();
  }

  public Pacient(Database database, String ID) {
    this(database, database.getLoader().getDefaultPacientLoader(), ID);
  }

  public Pacient(Database database, PacientLoader loader, String ID) {
    this(database, loader);
    this.ID = ID;
    loader.load(this, ID);
  }

  /**
   * vrt seznam nvtv
   * @return List
   */
  public List<Visit> getVisits() {
    List<Visit> v = (this.visits == null) ? null :
        (List<Visit>) (this.visits.get());
    if (v == null) {
      v = loader.getVisits(this);
      this.sortVisits(v);
      this.visits = new SoftReference(v);
    }
    return v;
  }

  public int visitCount() {
    return this.getVisits().size();
  }

  public Visit getVisit(int index) {
    return this.getVisits().get(index);
  }

  public int getIndex(Visit visit) {
    return this.getVisits().indexOf(visit);
  }

  public String getID() {
    return ID;
  }

  /**
   * pokus se nastavit nov ID
   * @param ID String
   */
  public void setID(String ID) {
    if (this.ID.equals("")) {
      this.ID = ID;
    }
    else {
      if (!database.contains(ID)) {
        if (loader.replace(this, this.ID, ID)) {
          if (database.contains(this.ID)) {
            database.replace(this.ID, ID);
          }
          this.ID = ID;
        }
      }
      else if (!database.getPacient(this.getLongID()).equals(this)) {
        System.err.println("Chyba zmeny ID pacienta - ID je jiz pouzite");
      }
    }
  }

  public long getLongID() {
    long h = 0;
    try {
      h = Long.parseLong(ID);
    }
    catch (Exception e) {}
    return h;
  }

  public void setFirstname(String name) {
    this.firstname = name;
  }

  public String getFirstname() {
    return firstname;
  }

  public void setSurname(String name) {
    this.surname = name;
  }

  public void setRightHanded(boolean rhp) {
    this.rhp = rhp;
  }

  public String getSurname() {
    return surname;
  }

  public void setBornDate(Date date) {
    this.born = date;
  }

  public Date getBornDate() {
    return born;
  }

  /**
   * ulo pacienta, pokud to lze
   * @return boolean
   */
  public boolean save() {
    if ( ((surname.trim().compareTo("") == 0) &&
        (firstname.trim().compareTo("") == 0)) || (this.getLongID() == 0)) {
      return false;
    }

    database.addPacient(this);
    loader.save(this);
    return true;
  }

  /**
   * pid nvtvu, pokud lze
   * @param visit Visit
   */
  public void addVisit(Visit visit) {
    if (!this.getVisits().contains(visit)) {
      this.getVisits().add(visit);
      this.sortVisits(this.getVisits());
    }
  }

  /**
   * vrac posledn nvtvu
   * @return Visit
   */
  public Visit getLastVisit() {
    Visit res = null;
    if (this.getVisits().size() > 0) {
      res = this.getVisits().get(0);
      Date last = this.getVisits().get(0).getVisitDate();
      for (Visit v : this.getVisits()) {
        if (v.getVisitDate().compareTo(last) > 0) {
          res = v;
        }
      }
    }
    return res;
  }

  /**
   * vrt nvtevu identifikovanou stejnm datem
   * @param visit Visit
   * @return Visit
   */
  public Visit visitExists(Visit visit) {
    int i = this.getIndex(visit);
    if (i > -1) {
      return this.getVisit(i);
    }
    return visit;
  }

  public String toString() {
    return surname + " " + firstname;
  }

  public boolean equals(Object o) {
    if (o == null) {
      return false;
    }
    if (o instanceof Pacient) {
      Pacient p = (Pacient) o;
      return this.ID.equals(p.ID);
    }
    return false;
  }

  public int hashCode() {
    return (int) (getLongID() % Integer.MAX_VALUE);
  }

  public void setDatabase(Database database) {
    this.database = database;
    this.loader = database.getLoader().getDefaultPacientLoader();
  }

  public void setLoader(PacientLoader loader) {
    this.loader = loader;
  }

  public Database getDatabase() {
    return database;
  }

  public boolean isRightHanded() {
    return rhp;
  }

  public PacientLoader getLoader() {
    return loader;
  }

  /**
   * sead nvtevy podle data
   * @param v List
   */
  private static void sortVisits(List<Visit> v) {
    Collections.sort(v, new Comparator<Visit> () {
      public int compare(Visit v1, Visit v2) {
        return v1.getVisitDate().compareTo(v2.getVisitDate());
      }
    });
  }

  /**
   * zanonymizuje pacienta
   * @param newID String nov ID a jmno
   * @return Pacient
   */
  public Pacient getAnonymus(String newID) {
    Pacient p = new Pacient(this);
    p.ID = newID;
    p.surname = newID;
    p.firstname = "";
    return p;
  }
}
