/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.database;

import java.util.*;
import spirals.motol.database.*;

/**
 * <p>Class contains information about tests performed by one person</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class TestList {
  private static List<Test> tests = null;
  private int hist[];

  public TestList() {
    if (tests == null) {
      tests = Engine.getInstance().getConfiguration().getTests();
    }
    int max = tests.size();
    for (Test t : tests) {
      if (t.getId() >= max) {
        max = t.getId() + 1;
      }
    }
    hist = new int[max];
  }

  /**
   * pidej do histogramu zez pro dan test
   * @param desc String
   */
  public void addHist(String desc) {
    for (Test t : tests) {
      if (t.getName().equals(desc)) {
        hist[t.getId()]++;
        break;
      }
    }
  }

  /**
   * vra histogram danho testu
   * @param desc String
   * @return int
   */
  public int getHist(String desc) {
    for (Test t : tests) {
      if (t.getName().equals(desc)) {
        return hist[t.getId()];
      }
    }
    return -1;
  }

  /**
   * data byla pejmenovna -> zm histogram
   * @param oldName String
   * @param newName String
   */
  public void updateHist(String oldName, String newName) {
    for (Test t : tests) {
      if (t.getName().equals(newName)) {
        hist[t.getId()]++;
      }
      if (t.getName().equals(oldName)) {
        hist[t.getId()]--;
      }
    }
  }

  /**
   * vra test s danm jmnem
   * @param desc String
   * @return Test
   */
  public static Test get(String desc) {
    for (Test t : tests) {
      if (t.getName().equals(desc)) {
        return t;
      }
    }
    return null;
  }

  /**
   * vra seznam zobrazovanch test
   * @return List
   */
  public static List<String> getShowable() {
    List<String> res = new ArrayList<String> (tests.size());
    for (Test t : tests) {
      if (t.isVisible()) {
        res.add(t.getName());
      }
    }
    return res;
  }

  /**
   * vrat seznam vech test
   * @return List
   */
  public static List<String> getAll() {
    List<String> res = new ArrayList<String> (tests.size());
    for (Test t : tests) {
      res.add(t.getName());
    }
    return res;
  }
}
