/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.database;

import spirals.motol.loaders.VisitLoader;
import java.util.*;
import java.text.*;

/**
 * <p>Class representing one visit at doctor for pacient.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class Visit {
  /**
   * datum nvtvy
   */
  private Date date = null;
  /**
   * pacient, ke ktermu nvtva pat
   */
  private Pacient pacient = null;
  /**
   * tda pro nahrn z databze
   */
  private VisitLoader loader = null;
  /**
   * seznam nasnmanch dat
   */
  private List<Data> data = null;
  /**
   * histogram test
   */
  private TestList tests = null;
  private String memo = "";
  private String diagnose = "";
  private boolean valid = false;
  int restTremorLeft = 0;
  int restTremorRight = 0;
  int posturalTremorLeft = 0;
  int posturalTremorRight = 0;
  int actionTremorLeft = 0;
  int actionTremorRight = 0;
  int openingLeft = 0;
  int openingRight = 0;
  int tappingLeft = 0;
  int tappingRight = 0;

  public Visit(Pacient pacient, Visit visit) {
    this(pacient, pacient.getLoader().getDefaultVisitLoader());
    if (visit != null) {
      this.memo = visit.getMemo();
      this.diagnose = visit.getDiagnose();
      this.valid = visit.isValid();
      this.restTremorLeft = visit.getRestTremorLeft();
      this.restTremorRight = visit.getRestTremorRight();
      this.posturalTremorLeft = visit.getPosturalTremorLeft();
      this.posturalTremorRight = visit.getPosturalTremorRight();
      this.actionTremorLeft = visit.getActionTremorLeft();
      this.actionTremorRight = visit.getActionTremorRight();
      this.openingLeft = visit.getOpeningLeft();
      this.openingRight = visit.getOpeningRight();
      this.tappingLeft = visit.getTappingLeft();
      this.tappingRight = visit.getTappingRight();
    }
  }

  public Visit(Pacient pacient) {
    this(pacient, pacient.getLoader().getDefaultVisitLoader());
  }

  private Visit(Pacient pacient, VisitLoader loader) {
    this.pacient = pacient;
    this.loader = loader;
    tests = new TestList();
    this.date = new Date();
    this.data = new ArrayList(0);
  }

  public Visit(Pacient pacient, Date date) {
    this(pacient, pacient.getLoader().getDefaultVisitLoader(), date);
  }

  private Visit(Pacient pacient, VisitLoader loader, Date date) {
    this(pacient, loader);
    this.date = date;
    loader.load(this, pacient, date);
  }

  public String toString() {
    SimpleDateFormat df = new SimpleDateFormat("dd. MM. yyyy, HH:mm");
    return df.format(date);
  }

  public boolean equals(Object o) {
    if (o == null) {
      return false;
    }
    if (o instanceof Visit) {
      Visit v = (Visit) o;
      return this.getPacient().equals(v.getPacient()) &&
          this.toString().equals(v.toString());
    }
    return false;
  }

  public Pacient getPacient() {
    return pacient;
  }

  public void setPacient(Pacient pacient) {
    this.pacient = pacient;
    this.loader = pacient.getLoader().getDefaultVisitLoader();
  }

  public void setLoader(VisitLoader loader) {
    this.loader = loader;
  }

  public List<Data> getData() {
    return data;
  }

  public int getDataSize() {
    return data.size();
  }

  public Date getVisitDate() {
    return date;
  }

  public String getMemo() {
    return memo;
  }

  public String getDiagnose() {
    return diagnose;
  }

  public boolean isValid() {
    return valid;
  }

  public void setMemo(String memo) {
    this.memo = memo;
  }

  public void setDiagnose(String diagnose) {
    this.diagnose = diagnose;
  }

  public void setValid(boolean valid) {
    this.valid = valid;
  }

  public void setVisitDate(Date date) {
    this.date = date;
  }

  public int getRestTremorLeft() {
    return restTremorLeft;
  }

  public int getRestTremorRight() {
    return restTremorRight;
  }

  public int getPosturalTremorLeft() {
    return posturalTremorLeft;
  }

  public int getPosturalTremorRight() {
    return posturalTremorRight;
  }

  public int getActionTremorLeft() {
    return actionTremorLeft;
  }

  public int getActionTremorRight() {
    return actionTremorRight;
  }

  public int getOpeningLeft() {
    return openingLeft;
  }

  public int getOpeningRight() {
    return openingRight;
  }

  public int getTappingLeft() {
    return tappingLeft;
  }

  public int getTappingRight() {
    return tappingRight;
  }

  public void setRestTremorLeft(int value) {
    restTremorLeft = value;
  }

  public void setRestTremorRight(int value) {
    restTremorRight = value;
  }

  public void setPosturalTremorLeft(int value) {
    posturalTremorLeft = value;
  }

  public void setPosturalTremorRight(int value) {
    posturalTremorRight = value;
  }

  public void setActionTremorLeft(int value) {
    actionTremorLeft = value;
  }

  public void setActionTremorRight(int value) {
    actionTremorRight = value;
  }

  public void setOpeningLeft(int value) {
    openingLeft = value;
  }

  public void setOpeningRight(int value) {
    openingRight = value;
  }

  public void setTappingLeft(int value) {
    tappingLeft = value;
  }

  public void setTappingRight(int value) {
    tappingRight = value;
  }

  public void save() {
    pacient.addVisit(this);
    loader.save(this);
  }

  /**
   * pidej data, pokud lze a uprav histogram
   * @param data Data
   */
  public void addData(Data data) {
    if (!this.data.contains(data)) {
      this.data.add(data);
      tests.addHist(data.getDescription());
    }
  }

  public TestList getTestList() {
    return tests;
  }

  public VisitLoader getLoader() {
    return loader;
  }
}
