/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.gui;

import spirals.motol.database.*;
import spirals.data.*;
import spirals.data.viewers.SimpleTabletDataViewer;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import tablet.*;
import java.util.*;
import java.awt.image.*;

/**
 * <p>Frame where data from tablet are catured and saved.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class CaptureFrame
    extends JDialog implements TabletListener {
  /**
   * old x position
   */
  private int ox;
  /**
   * old y position
   */
  private int oy;
  /**
   * tablet pen is touching surface
   */
  private boolean pen_down = false;
  private boolean show;
  /**
   * ekn na prvn paket
   */
  protected boolean firstPacket;
  /**
   * as zatku zznamu
   */
  protected long startTime;
  /**
   * tablet data viewer
   */
  protected SimpleTabletDataViewer viewer;

  /**
   * visit for which are data captured
   */
  protected Visit visit;
  /**
   * contains contents of frame
   */
  private JPanel contentPane;
  /**
   * actual tablet measures
   */
  private TabletInfo tabletInfo;
  /**
   * picture for already drawn type of data
   */
  private Icon okIcon = new ImageIcon(CaptureFrame.class.getResource(
      "/res/ok.gif"));
  /**
   * engine
   */
  private Engine engine;
  /**
   * capturing tablet data
   */
  private Data data;
  private TabletData tabletData;

  /**
   * some tablet data ready
   */
  protected boolean dataReady = false;
  /**
   * button performing start of capturing
   */
  protected JButton startButton;

  /**
   * buttons for closing frame
   */
  private JButton exitButton = new JButton(new Capture_Exit_Action(this));
  /**
   * list of types of drawings
   */
  private ButtonGroup type = new ButtonGroup();
  /**
   * container for types of drawings
   */
  private JPanel leftCenter;

  /**
   * panel for placing controls of viewer panel
   */
  private JPanel control = new JPanel();
  /**
   * area for view drawing
   */
  protected JPanel screen = new JPanel() {
    public Dimension getPreferredSize() {
      return new Dimension(600, 400);
    }

    public void paintComponent(Graphics g) {
      viewer.paint(g);
    }
  };

  /**
   * vlkno pro potn dlky snmn
   */
  protected TimeThread time;

  /**
   * inicializuj nov data
   * @return Data
   */
  private Data getNewData() {
    Data data = new Data(visit);
    tabletData = data.getAllData();
    return data;
  }

  /**
   * create frame for visit
   */
  public CaptureFrame(Engine engine, Visit visit) {
    super(engine.getMainFrame(), "Zznam dat z tabletu", true);
    this.engine = engine;
    this.visit = visit;
    this.tabletInfo = engine.getTablet().getTabletInfo();
    this.data = this.getNewData();
    this.viewer = new SimpleTabletDataViewer(screen, control, data, true, true);

    this.contentPane = (JPanel)this.getContentPane();
    this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    this.addWindowListener(new Capture_Exit_Action(this));
    this.createLayout();
    this.colorize();
  }

  /**
   * create and place frame components
   */
  private void createLayout() {
    JPanel bottom = new JPanel();
    JPanel center = new JPanel(new BorderLayout());
    startButton = new Capture_Start_Button(engine, this);
    bottom.add(startButton);
    bottom.add(Box.createHorizontalStrut(10));
    bottom.add(exitButton);

    JPanel right = new JPanel(new BorderLayout());
    right.add(screen, BorderLayout.CENTER);
    right.add(control, BorderLayout.SOUTH);
    center.add(right, BorderLayout.CENTER);
    center.add(bottom, BorderLayout.SOUTH);

    contentPane.add(createLeftPanel(), BorderLayout.WEST);
    contentPane.add(center, BorderLayout.CENTER);
  }

  /**
   * create and return panel with types of drawings
   */
  private JPanel createLeftPanel() {
    JPanel left = new JPanel(new BorderLayout());
    java.util.List<String> types = visit.getTestList().getShowable();
    leftCenter = new JPanel(new GridLayout(types.size(), 1));
    leftCenter.setBorder(BorderFactory.createEtchedBorder());
    for (int i = 0; i < types.size(); i++) {
      JPanel row = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
      JRadioButton radio = new JRadioButton(types.get(i).toString());
      radio.setActionCommand(types.get(i).toString());
      type.add(radio);
      row.add(radio);
      row.add(new JLabel());
      leftCenter.add(row);
    }

    left.add(leftCenter, BorderLayout.NORTH);
    return left;
  }

  /**
   * volat pi vypren asu
   */
  public void onTimeout() {
    JOptionPane.showMessageDialog(engine.getMainFrame(),
                                  "Snmn dat pekroilo maximln povolenou dlku " +
                                  engine.getConfiguration().getTimeout() +
                                  " vtein. Snmn bylo ukoneno",
                                  "Chyba",
                                  JOptionPane.ERROR_MESSAGE);
    startButton.setText("Start");
    this.setButtonsEnabled(true);
    this.data = this.getNewData();
    viewer.setTabletData(this.data);
    screen.repaint();
    this.setRadioEnabled(true);
    this.dataReady = false;
  }

  /**
   * called by tablet when tablet packet is ready
   * <p>Add packet to data sequence add display new point.</p>
   */
  public void onTabletPacket(TabletPoint point) {
    if (firstPacket) {
      // pi prvnm paketu spus potadlo
      firstPacket = false;
      startTime = System.currentTimeMillis();
      time = new TimeThread(this);
      time.start();
    }
    this.dataReady = true;
    tabletData.add(point);
    // view and tablet area measures
    Graphics g = screen.getGraphics();
    int scr_width = screen.getWidth();
    int scr_height = screen.getHeight();
    int tab_width = tabletInfo.getWidth();
    int tab_height = tabletInfo.getHeight();

    // which dimension have bigger compress ratio
    double r_w = ( (float) scr_width) / (float) tab_width;
    double r_h = ( (float) scr_height) / (float) tab_height;
    double koef = (r_h > r_w) ? r_w : r_h;

    int x = (int) (point.getX() * koef);
    int y = (int) (scr_height - point.getY() * koef);

    if (!point.isTouched() && show) {
      pen_down = false;
      return;
    }

    if (!pen_down) { // beginning of new line
      pen_down = true;
      ox = x;
      oy = y;
    }

    g.setColor(Color.black);
    g.drawLine(x, y, ox, oy);
    ox = x;
    oy = y;
  }

  /**
   * return selected type of drawing
   */
  protected String getType() {
    if(type.getSelection() != null) return type.getSelection().getActionCommand();
    else return null;
  }

  /**
   * start capturing form tablet
   */
  protected void startCapture() {
    setButtonsEnabled(false);
    viewer.disableControls();
    this.data = this.getNewData();
    pen_down = false;
    Test t = visit.getTestList().get(getType());
    viewer.setTabletData(data);
    firstPacket = true;
    show = t.showUp();
    engine.getTablet().startCapture(t.startByStart());

    if (time != null) time.end();
  }

  /**
   * colorize all already captured types of drawings and set next unused
   */
  private void colorize() {
    Component[] left = leftCenter.getComponents();
    TestList tests = visit.getTestList();
    // colorize all used types
    for (int i = 0; i < left.length; i++) {
      Component[] list = ( (JPanel) left[i]).getComponents();
      AbstractButton button = ( (AbstractButton) list[0]);
      int num = tests.getHist(button.getText());
      if (num > 0) {
        button.setForeground(Color.GREEN);
        ( (JLabel) list[1]).setText(" (" + num + ")");
        ( (JLabel) list[1]).setIcon(okIcon);
      }
    }
    // set next unused type
    for (int i = 0; i < left.length; i++) {
      Component[] list = ( (JPanel) left[i]).getComponents();
      AbstractButton button = ( (AbstractButton) list[0]);
      int num = tests.getHist(button.getText());
      if (num == 0) {
        ( (AbstractButton) ( (JPanel) left[i]).getComponents()[0]).setSelected(true);
        return;
      }
    }
    ( (AbstractButton) ( (JPanel) left[0]).getComponents()[0]).setSelected(true);
  }

  /**
   * save captured data to disk
   */
  protected void save() {
    if (visit.getTestList().get(getType()).trim()) {
      tabletData.trim();
    }
    data.setDate(new Date());
    data.setDescription(getType());

    // otevi okno pro oznaen sprvnho seku dat
    EditDataFrame dlg = new EditDataFrame(engine.getMainFrame(), visit, data, true);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = this.getSize();
    Point loc = this.getLocation();
    dlg.setLocation( (frmSize.width - dlgSize.width) / 2 + loc.x,
                    (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.pack();
    dlg.setVisible(true);

    // pekresli a jedem znova
    this.colorize();
    startButton.setText("Start");
    this.data = this.getNewData();
    viewer.setTabletData(this.data);
    screen.repaint();
    this.setRadioEnabled(true);
    this.dataReady = false;
  }

  /**
   * called at beggining and end of capturing.
   */
  protected void setButtonsEnabled(boolean enabled) {
    exitButton.setEnabled(enabled);
    if (!enabled) {
      this.setRadioEnabled(enabled);
    }
  }

  /**
   * enables/disables types of drawings
   */
  private void setRadioEnabled(boolean enabled) {
    Enumeration e = type.getElements();
    while (e.hasMoreElements()) {
      ( (AbstractButton) e.nextElement()).setEnabled(enabled);
    }
  }

  /**
   * Escape zave okno
   * @param ke KeyEvent
   */
  public void keyReleased(KeyEvent ke) {
    if(ke.getKeyCode() == KeyEvent.VK_ESCAPE) {
      if(exitButton.isEnabled()) exitButton.doClick();
    }
  }
  public void keyPressed(KeyEvent ke) {}
  public void keyTyped(KeyEvent ke) {}
}

/**
 * <p>Button performing starting and ending of capturing data for CaptureFrame.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class Capture_Start_Button
    extends JButton implements ActionListener {
  /**
   * engine
   */
  Engine engine;
  /**
   * parent CaptureFrame
   */
  CaptureFrame adaptee;

  /**
   * create button for CaptureFrame
   */
  Capture_Start_Button(Engine engine, CaptureFrame adaptee) {
    super("Start");
    this.engine = engine;
    this.adaptee = adaptee;
    this.addActionListener(this);
    this.setMnemonic(KeyEvent.VK_S);
  }

  /**
   * called when start/stop button is pressed
   */
  public void actionPerformed(ActionEvent actionEvent) {
    if (!this.getText().equals("Stop")) { // start capturing
      this.setText("Stop");
      adaptee.startCapture();
    }
    else { // end capturing
      this.setText("Znovu");
      engine.getTablet().stopCapture();
      adaptee.setButtonsEnabled(true);
      adaptee.viewer.updateControls();
      if(adaptee.time != null) adaptee.time.end();
      if (adaptee.dataReady) {
        adaptee.save();
      }
    }
  }
}

/**
 * <p>Action called when frame is being closed.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class Capture_Exit_Action
    extends AbstractAction implements WindowListener {
  /**
   * parent CaptureFrame
   */
  CaptureFrame adaptee;

  /**
   * create action for CaptureFrame
   */
  Capture_Exit_Action(CaptureFrame adaptee) {
    super("Ukoni");
    this.adaptee = adaptee;
  }

  /**
   * Need for sake of Java
   */
  public void windowActivated(WindowEvent e) {}

  /**
   * Need for sake of Java
   */
  public void windowClosed(WindowEvent e) {}

  /**
   * Need for sake of Java
   */
  public void windowDeactivated(WindowEvent e) {}

  /**
   * Need for sake of Java
   */
  public void windowDeiconified(WindowEvent e) {}

  /**
   * Need for sake of Java
   */
  public void windowIconified(WindowEvent e) {}

  /**
   * Need for sake of Java
   */
  public void windowOpened(WindowEvent e) {}

  /**
   * called when closing window
   */
  public void windowClosing(WindowEvent e) {
    this.exit();
  }

  /**
   * called when close button clicked
   */
  public void actionPerformed(ActionEvent e) {
    this.exit();
  }

  private void exit() {
    if (!adaptee.startButton.getText().equals("Stop")) {
      adaptee.dispose();
    }
  }
};

/**
 * <p>Tda pro sledovn asu snmn.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class TimeThread
    extends Thread {
  private boolean running;
  private CaptureFrame frame;
  private BufferedImage img = null;
  private int width;
  private int height;
  private Font font = new Font("Tahoma", Font.PLAIN, 20);

  public TimeThread(CaptureFrame cf) {
    frame = cf;
    FontMetrics fm = cf.getFontMetrics(font);
    width = fm.stringWidth("8888");
    height = fm.getHeight();
    img = new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR);
  }

  public void run() {
    running = true;
    while (running) {
      long sec = (System.currentTimeMillis() - frame.startTime) / 1000;
      if (!frame.firstPacket) {
        long min = sec / 60;
        Graphics g = frame.screen.getGraphics();
        Graphics imgg = img.getGraphics();
        imgg.setColor(Color.WHITE);
        imgg.fillRect(0, 0 , width, height);
        imgg.setColor(Color.RED);
        imgg.setFont(font);
        imgg.drawString(String.format("%d:%02d", min, sec % 60), 0, height);
        g.drawImage(img, 6, 6, width, height, null);
      }
      try {
        sleep(1000);
      }
      catch (Exception e) {}
    }
  }

  public synchronized void end() {
    running = false;
  }
}
