/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.gui;

import spirals.motol.database.*;
import tablet.TabletInfo;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.UIManager.*;
import java.util.*;

/**
 * <p>Frame for application configurtion.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class ConfigFrame extends JDialog {
  /**
   * engine
   */
  protected Engine engine;
  /**
   * contains contents of frame
   */
  private JPanel contentPane;
  /**
   * main content of frame
   */
  private JPanel tabbedPane = new JPanel(new BorderLayout());
  /**
   * list of available look and feel classes
   */
  protected LookAndFeelInfo[] lafList = UIManager.getInstalledLookAndFeels();
  /**
   * list of apearances
   */
  protected JComboBox uimanager;
  /**
   * list of tablets
   */
  protected JComboBox tablets;

  /**
   * create config dialog frame
   */
  public ConfigFrame(Engine engine) {
    super(engine.getMainFrame(), "Konfigurace", true);
    this.engine = engine;
    this.contentPane = (JPanel) this.getContentPane();
    JPanel bottom = new JPanel();
    bottom.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    bottom.add(new JButton(new AbstractAction("Zru") {
      public void actionPerformed(ActionEvent e) {
        ConfigFrame.this.dispose();
      }
    }));
    bottom.add(Box.createHorizontalStrut(10));
    bottom.add(new JButton(new Config_OK_Action(this)));
    contentPane.add(tabbedPane, BorderLayout.CENTER);
    contentPane.add(bottom, BorderLayout.SOUTH);
    this.createPane();
  }

  /**
   * create content for frame
   */
  private void createPane() {
    JPanel top = new JPanel(new BorderLayout());
    top.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    JPanel gui = new JPanel(new GridBagLayout());
    String[] list = new String[lafList.length];
    for(int i = 0; i < lafList.length; i++) {
      list[i] = lafList[i].getName();
    }
    uimanager = new JComboBox(list);
    uimanager.setSelectedItem(UIManager.getLookAndFeel().getName());
    java.util.List tabs = engine.getConfiguration().getAvailableTablets();
    tablets = new JComboBox(tabs.toArray());
    tablets.setSelectedIndex(engine.getConfiguration().getSelectedTabletIndex());
    this.place(gui, 0, new JLabel("Vzhled"), uimanager);
    this.place(gui, 1, new JLabel("Vyber tablet"), tablets);
    top.add(gui, BorderLayout.NORTH);
    tabbedPane.add(top, BorderLayout.CENTER);
  }

  /**
   * place two components in gridbag layout
   * @param panel JPanel, where components are placed
   * @param row row number in parent component
   * @param c1 first component to place
   * @param c2 second component to place
   */
  private void place(JPanel panel, int row, JComponent c1, JComponent c2) {
    panel.add(c1, new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0, 0, 0, 10), 0, 0));
    panel.add(c2, new GridBagConstraints(1, row, 2, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    c1.setFont(engine.getMainFrame().getDefaultBoldFont());
    c2.setFont(engine.getMainFrame().getDefaultBoldFont());
  }
}

/**
 * <p>Action performed when OK is pressed in configuration dialog.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 *
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.1
 */
class Config_OK_Action extends AbstractAction {
  /**
   * parent ConfigFrame
   */
  ConfigFrame adaptee;

  /**
   * create action for ConfigFrame
   */
  public Config_OK_Action(ConfigFrame adaptee) {
    super("OK");
    this.adaptee = adaptee;
  }

  /**
   * OK button is pressed
   */
  public void actionPerformed(ActionEvent e) {
    String lafname = (String) adaptee.uimanager.getSelectedItem();
    for(final LookAndFeelInfo laf : adaptee.lafList) {
      if(laf.getName().equals(lafname)) {
        // laf class is selected
        adaptee.engine.getConfiguration().setLafClass(laf.getClassName());
        // set new look and feel class
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            try {
              UIManager.setLookAndFeel(laf.getClassName());
              SwingUtilities.updateComponentTreeUI(adaptee.engine.getMainFrame());
            } catch(Exception e) {
              e.printStackTrace();
            }
          }
        });
      }
    }
    adaptee.engine.selectTablet((TabletInfo)adaptee.tablets.getSelectedItem());
    adaptee.engine.getConfiguration().save();
    adaptee.dispose();
  }
}
