/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.gui;

import spirals.data.*;
import spirals.data.viewers.*;
import spirals.motol.database.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * <p>Frame for viewing captured data from tablet.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class DataViewerFrame extends JFrame implements ActionListener, KeyListener {
  /**
   * data viewver class
   */
  TabletDataViewer viewer;
  Data data;
  /**
   * contains content of frame
   */
  JPanel contentPane;

  /**
   * area for view drawing
   */
  JPanel screen = new JPanel() {
    public Dimension getPreferredSize() {
      return new Dimension(600,400);
    }

    public void paintComponent(Graphics g) {
      viewer.paint(g);
    }
  };
  JPanel bottom = new JPanel();
  JTextArea description = new JTextArea();
  ViewersComboBox chooser = new ViewersComboBox();
  boolean showUp = true;

  /**
   * create and lay out new viewer frame for Data
   */
  public DataViewerFrame(Visit visit, Data data) {
    super(data.toString());
    this.data = data;
    Test t = visit.getTestList().get(data.getDescription());
    bottom.setBorder(BorderFactory.createEmptyBorder(10,0,0,0));
    showUp = t.showUp();
    this.actionPerformed(null);
    this.contentPane = (JPanel) this.getContentPane();
    contentPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    this.add(screen, BorderLayout.CENTER);
    this.add(bottom, BorderLayout.SOUTH);
    JPanel right = new JPanel(new BorderLayout());
    chooser.addActionListener(this);
    right.setBorder(BorderFactory.createEmptyBorder(0,10,0,0));
    right.add(chooser, BorderLayout.NORTH);
    description.setBorder(BorderFactory.createEmptyBorder(10,5,10,5));
    description.setEditable(false);
    description.setLineWrap(true);
    description.setWrapStyleWord(true);
    right.add(description, BorderLayout.CENTER);
    right.add(new JButton(new AbstractAction("Zavi") {
      public void actionPerformed(ActionEvent e) {
        dispose();
      }
    }), BorderLayout.SOUTH);
    this.add(right, BorderLayout.EAST);
    this.addKeyListener(this);
  }

  public void actionPerformed(ActionEvent ae) {
    this.viewer = chooser.invokeClass(screen, bottom, data, false, showUp);
    this.description.setText(this.viewer.getDescription());
    this.validate();
    screen.repaint();
  }

  public void keyReleased(KeyEvent ke) {
    if(ke.getKeyCode() == KeyEvent.VK_ESCAPE) {
      this.dispose();
    }
  }
  public void keyPressed(KeyEvent ke) {}
  public void keyTyped(KeyEvent ke) {}
}
