/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.gui;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import spirals.data.doubleViewers.TimeTabletDataViewer;
import spirals.motol.database.*;
import tablet.*;

/**
 * <p>Frame for editing captured data.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class EditDataFrame extends JDialog implements KeyListener {
  /**
   * views captured data
   */
  protected TimeTabletDataViewer viewer;
  /**
   * contains content of frame
   */
  private JPanel contentPane;
  /**
   * performs clean exit
   */
  private ExitListener ea = new ExitListener(this);
  /**
   * visit which data belongs to
   */
  protected Visit visit;
  /**
   * data for editing
   */
  protected final Data data;
  /**
   * data has been changed
   */
  protected boolean changed = false;
  protected boolean capturing = false;
  /**
   * list of available types of drawings
   */
  protected JComboBox type;
  /**
   * list of available tablets
   */
  protected JComboBox tablet;
  /**
   * comment for data
   */
  protected JEditorPane memo = new JEditorPane();

  /**
   * area where is performed drawing
   */
  private JPanel screen = new JPanel() {
    public Dimension getPreferredSize() {
      return new Dimension(400,300);
    }

    public void paintComponent(Graphics g) {
      viewer.paint(g);
    }
  };

  /**
   * create edit frame for Data in Visit
   */
  public EditDataFrame(JFrame parent, Visit visit, Data data, boolean capturing) {
    super(parent, data.toString(), true);
    this.setPreferredSize(new Dimension(800, 600));
    this.visit = visit;
    this.data = data;
    this.capturing = capturing;
    Test t = visit.getTestList().get(data.getDescription());
    JPanel control = new JPanel();
    this.viewer = new TimeTabletDataViewer(screen, control, data, true, t.showUp());
    viewer.setChanged(capturing);
    this.contentPane = (JPanel) this.getContentPane();
    this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    this.addWindowListener(ea);
    contentPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    JPanel right = new JPanel(new BorderLayout());
    right.setBorder(BorderFactory.createEmptyBorder(0,10,10,0));

    JPanel top = new JPanel(new GridLayout(4,1));
    // type of drawing
    type = new JComboBox(visit.getTestList().getAll().toArray());
    type.setSelectedItem(data.getDescription());
    type.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        changed = true;
      }
    });
    top.add(new JLabel("Typ:"));
    top.add(type);

    // tablet
    tablet = new JComboBox(Engine.getInstance().getConfiguration().getAvailableTablets().toArray());
    tablet.setSelectedItem(data.getTabletInfo());
    tablet.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        changed = true;
//        data.setTabletInfo((TabletInfo) tablet.getSelectedItem());
      }
    });
    top.add(new JLabel("Tablet:"));
    top.add(tablet);
    top.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));

    // memo
    JPanel m = new JPanel(new GridBagLayout());
    memo.setText(data.getMemo());
    memo.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent e) {
        changed = true;
      }
    });
    m.setBorder(BorderFactory.createEmptyBorder(0,0,20,0));
    m.add(new JLabel("Poznmka:"), new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0, 0, 0, 10), 0, 0));
    m.add(new JScrollPane(memo), new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 10), 0, 0));
    right.add(m, BorderLayout.CENTER);

    // buttons
    JPanel buttons = new JPanel();
    buttons.add(new JButton(new AbstractAction("Ulo") {
      public void actionPerformed(ActionEvent e) {
        save();
      }
    }));
    buttons.add(new JButton(new AbstractAction("Zavi") {
      public void actionPerformed(ActionEvent e) {
        ea.windowClosing(null);
      }
    }));
    right.add(buttons, BorderLayout.SOUTH);
    right.add(top, BorderLayout.NORTH);
    JPanel left = new JPanel(new BorderLayout());
    left.add(screen, BorderLayout.CENTER);
    left.add(control, BorderLayout.SOUTH);
    this.add(left, BorderLayout.CENTER);
    this.add(right, BorderLayout.EAST);
    this.addKeyListener(this);
  }

  public void keyReleased(KeyEvent ke) {
    if(ke.getKeyCode() == KeyEvent.VK_ESCAPE) {
      ea.windowClosing(null);
    }
  }
  public void keyPressed(KeyEvent ke) {}
  public void keyTyped(KeyEvent ke) {}

  public void save() {
    data.setDescription((String) type.getSelectedItem());
    data.setTabletInfo((TabletInfo) tablet.getSelectedItem());
    data.setMemo(memo.getText());
    int start = Math.min(viewer.getStart(), viewer.getEnd());
    int end = Math.max(viewer.getStart(), viewer.getEnd());
    data.setStart(start);
    data.setEnd(end);
    data.save(capturing);
    viewer.setChanged(false);
    this.changed = false;
  }
}

/**
 * <p>Class performs clean exit from EditDataFrame</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class ExitListener extends WindowAdapter {
  /**
   * parent EditDataFrame
   */
  private EditDataFrame adaptee;

  /**
   * create listener for EditDataFrame
   */
  public ExitListener(EditDataFrame adaptee) {
    this.adaptee = adaptee;
  }

  /**
   * executed on window closing
   */
  public void windowClosing(WindowEvent e) {
    if(adaptee.changed || adaptee.viewer.changed()) {
      Object[] options = {"Ano", "Ne"};
      String text = "daje o datech zmnny. Skonit bez uloen?";
      if(adaptee.capturing) text = "Data nebudou uloena. Pokraovat bez uloen?";
      int ret = JOptionPane.showOptionDialog(adaptee, text, "Data zmnna", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, options, options[0]);
      if(ret == JOptionPane.YES_OPTION) {
        adaptee.dispose();
//        adaptee.save();
      }
    } else {
      adaptee.dispose();
    }
  }
}
