/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.gui;

import javax.swing.*;
import javax.swing.event.*;
import spirals.motol.database.*;
import java.awt.*;
import java.awt.event.*;
import java.text.SimpleDateFormat;

/**
 * <p>Panel wit information about visit of pacient</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class VisitPanel extends RemovablePanel {
  /**
   * main frame
   */
  private MainFrame mf;
  /**
   * visit for this panel
   */
  private Visit visit;

  /**
   * listen for key events on form fields
   */
  private KeyListener changeAction = new KeyAdapter() {
    public void keyTyped(KeyEvent e) {
      VisitPanel.this.setChanged(true);
    }
  };

  /**
   * panel with information about visit
   */
  private JPanel visitPanel = new JPanel(new GridBagLayout());
  /**
   * panel with information about data
   */
  private JPanel dataPanel = new JPanel(new BorderLayout());
  /**
   * formatted text field for visit date
   */
  private JFormattedTextField visitDate = new JFormattedTextField(new SimpleDateFormat("dd. MM. yyyy, HH:mm"));
  /**
   * text box for diagnose
   */
  private JEditorPane diagnose;
  /**
   * text box for comment
   */
  private JEditorPane memo;
  /**
   * check box for valid diagnose
   */
  private JCheckBox valid = new JCheckBox("");
  /**
   * option for rest tremor in left hand
   */
  private MyCombo restTremorLeft;
  /**
   * option for rest tremor in right hand
   */
  private MyCombo restTremorRight;
  /**
   * option for rest postural in left hand
   */
  private MyCombo posturalTremorLeft;
  /**
   * option for rest postural in right hand
   */
  private MyCombo posturalTremorRight;
  /**
   * option for rest action in left hand
   */
  private MyCombo actionTremorLeft;
  /**
   * option for rest action in right hand
   */
  private MyCombo actionTremorRight;
  /**
   * option for opening of left hand
   */
  private MyCombo openingLeft;
  /**
   * option for opening of right hand
   */
  private MyCombo openingRight;
  /**
   * option for tapping of left hand
   */
  private MyCombo tappingLeft;
  /**
   * option for tapping of right hand
   */
  private MyCombo tappingRight;
  /**
   * list of data for visit
   */
  private JList dataList;
  /**
   * change record button
   */
  private JButton changeButton = new JButton("Zm daje");
  /**
   * new data button
   */
  private JButton newButton = new JButton("Nov data");
  /**
   * edit data button
   */
  private JButton editButton = new JButton("Uprav data");
  /**
   * show data button
   */
  private JButton showButton = new JButton("Uka data");
  private boolean changable;

  /**
   * create VisitPanel for Visit
   * @param frame MainFrame
   * @param visit Visit
   * @param changable user can change fields
   */
  public VisitPanel(MainFrame frame, final Visit visit, boolean changable) {
    super(new BorderLayout());
    this.mf = frame;
    this.visit = visit;
    this.install();
    this.changable = changable;
    this.setChanged(changable);
    this.setChangable(changable);
    if(changable) {
      this.saveData();
      SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          mf.getTree().getTree().updateUI();
          mf.getTree().scrollTreePath(visit);
        }
      });
    }
  }

  /**
   * lay out panel
   */
  private void install() {
    this.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
    final JTabbedPane mainContent = new JTabbedPane();
    mainContent.add("Diagnza", visitPanel);
    mainContent.add("Data", dataPanel);
    mainContent.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        notifyRemoved();
      }
    });

    this.createDiagnose();
    this.createData();
    this.add(mainContent, BorderLayout.CENTER);

    JPanel bottomPanel = new JPanel(new BorderLayout());
    changeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        changable = true;
        setChangable(changable);
      }
    });
    bottomPanel.add(changeButton, BorderLayout.WEST);
    bottomPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    this.add(bottomPanel, BorderLayout.SOUTH);
  }

  /**
   * create diagnose information panel
   */
  private void createDiagnose() {
    visitPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

    visitDate.setValue(visit.getVisitDate());
    visitDate.setEditable(false);
    diagnose = new JEditorPane("text/plain", visit.getDiagnose());
    diagnose.addKeyListener(changeAction);
    memo = new JEditorPane("text/plain", visit.getMemo());
    memo.addKeyListener(changeAction);
    valid.setSelected(visit.isValid());
    valid.addActionListener(new AbstractAction() {
      public void actionPerformed(ActionEvent e) {
        VisitPanel.this.setChanged(true);
      }
    });

    restTremorLeft = new MyCombo(this, visit.getRestTremorLeft());
    restTremorRight = new MyCombo(this, visit.getRestTremorRight());
    posturalTremorLeft = new MyCombo(this, visit.getPosturalTremorLeft());
    posturalTremorRight = new MyCombo(this, visit.getPosturalTremorRight());
    actionTremorLeft = new MyCombo(this, visit.getActionTremorLeft());
    actionTremorRight = new MyCombo(this, visit.getActionTremorRight());
    openingLeft = new MyCombo(this, visit.getOpeningLeft());
    openingRight = new MyCombo(this, visit.getOpeningRight());
    tappingLeft = new MyCombo(this, visit.getTappingLeft());
    tappingRight = new MyCombo(this, visit.getTappingRight());
    this.place(0, new JLabel("Datum nvtvy"), visitDate);
    this.place(1, new JLabel("Diagnza"), new JScrollPane(diagnose), 70);
    this.place(2, new JLabel("Validn"), valid);
    this.place(3, new JLabel("Poznmka"), new JScrollPane(memo), 60);
    this.place(4, new JLabel(" "), Box.createHorizontalBox(), Box.createHorizontalBox());
    this.place(5, Box.createHorizontalBox(), new JLabel(" LHK "), new JLabel(" PHK "));
    this.place(6, new JLabel("Tes klidov"), restTremorLeft, restTremorRight);
    this.place(7, new JLabel("Tes posturln"), posturalTremorLeft, posturalTremorRight);
    this.place(8, new JLabel("Tes intenn"), actionTremorLeft, actionTremorRight);
    this.place(9, new JLabel("Otvrn/zavrn ruky"), openingLeft, openingRight);
    this.place(10, new JLabel("Klepn prstem"), tappingLeft, tappingRight);
  }

  /**
   * place two components in gridbag layout on main panel
   * @param row row number in parent component
   * @param c1 first component to place
   * @param c2 second component to place
   */
  private void place(int row, JComponent c1, JComponent c2) {
    visitPanel.add(c1, new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0, 0, 0, 10), 0, 0));
    visitPanel.add(c2, new GridBagConstraints(1, row, 2, 1, 2.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    visitPanel.add(Box.createHorizontalBox(), new GridBagConstraints(3, row, 2, 1, 2.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
  }

  /**
   * place two components in gridbag layout on main panel with gap above
   * @param row row number in parent component
   * @param c1 first component to place
   * @param c2 second component to place
   * @param space pixels of gap
   */
  private void place(int row, JComponent c1, JComponent c2, int space) {
    visitPanel.add(c1, new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(space/3, 0, (space/3)*2, 10), 0, 0));
    visitPanel.add(c2, new GridBagConstraints(1, row, 4, 1, 4.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
  }

  /**
   * place three components in gridbag layout on main panel
   * @param row row number in parent component
   * @param c1 first component to place
   * @param c2 second component to place
   * @param c3 third component to place
   */
  private void place(int row, JComponent c1, JComponent c2, JComponent c3) {
    visitPanel.add(c1, new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0, 0, 0, 10), 0, 0));
    visitPanel.add(c2, new GridBagConstraints(1, row, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    visitPanel.add(c3, new GridBagConstraints(2, row, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    visitPanel.add(Box.createHorizontalBox(), new GridBagConstraints(3, row, 2, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
  }

  /**
   * create data list panel
   */
  private void createData() {
    dataList = new JList(visit.getData().toArray());
    dataList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    dataList.setFont(mf.getDefaultFont());
    dataList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        editButton.setEnabled(!changeButton.isEnabled());
        showButton.setEnabled(true);
      }
    });
    dataList.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if((e.getButton() == e.BUTTON1) && (e.getClickCount() > 1)) {
          showSelectedData();
        }
      }
    });
    dataPanel.add(new JScrollPane(dataList), BorderLayout.CENTER);
    JPanel buttons = new JPanel();
    buttons.setBorder(BorderFactory.createEmptyBorder(20,10,10,10));

    newButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Engine eng = visit.getPacient().getDatabase().getEngine();
        try {
          eng.getTablet().open();
        }
        catch(Error er) {
          JOptionPane.showMessageDialog(mf,
                              "Tablet nelze nalzt!",
                              "Chyba",
                              JOptionPane.ERROR_MESSAGE);
          return;
        }
        CaptureFrame cf = new CaptureFrame(eng, visit);
        Dimension dlgSize = cf.getPreferredSize();
        Dimension frmSize = mf.getSize();
        Point loc = mf.getLocation();
        cf.setLocation( (frmSize.width - dlgSize.width) / 2 + loc.x,
                       (frmSize.height - dlgSize.height) / 2 + loc.y);
        cf.pack();
        eng.getTablet().addTabletListener(cf);
        cf.setVisible(true);
        eng.getTablet().removeTabletListener(cf);
        dataList.setListData(visit.getData().toArray());
        setChangable(changable);
      }
    });
    newButton.setIcon(new ImageIcon(VisitPanel.class.getResource("/res/new.gif")));
    newButton.setVerticalTextPosition(SwingConstants.BOTTOM);
    newButton.setHorizontalTextPosition(SwingConstants.CENTER);
    buttons.add(newButton);
    buttons.add(Box.createHorizontalStrut(10));

    editButton.addActionListener(new AbstractAction("Uprav data") {
      public void actionPerformed(ActionEvent e) {
        if(dataList.getSelectedValue() != null) {
          EditDataFrame edf = new EditDataFrame(mf, visit, (Data) dataList.getSelectedValue(), false);
          Dimension dlgSize = edf.getPreferredSize();
          Dimension frmSize = mf.getSize();
          Point loc = mf.getLocation();
          edf.setLocation( (frmSize.width - dlgSize.width) / 2 + loc.x,
                          (frmSize.height - dlgSize.height) / 2 + loc.y);
          edf.pack();
          edf.setVisible(true);
        }
      }
    });
    editButton.setIcon(new ImageIcon(VisitPanel.class.getResource("/res/edit.gif")));
    editButton.setVerticalTextPosition(SwingConstants.BOTTOM);
    editButton.setHorizontalTextPosition(SwingConstants.CENTER);
    buttons.add(editButton);
    buttons.add(Box.createHorizontalStrut(10));

    showButton.addActionListener(new AbstractAction("Uka data") {
      public void actionPerformed(ActionEvent e) {
        showSelectedData();
      }
    });
    showButton.setIcon(new ImageIcon(VisitPanel.class.getResource("/res/image.gif")));
    showButton.setVerticalTextPosition(SwingConstants.BOTTOM);
    showButton.setHorizontalTextPosition(SwingConstants.CENTER);
    buttons.add(showButton);

    dataPanel.add(buttons, BorderLayout.SOUTH);
  }

  /**
   * set form fields and button changable
   */
  private void setChangable(boolean changable) {
    diagnose.setEnabled(changable);
    memo.setEnabled(changable);
    valid.setEnabled(changable);
    restTremorLeft.setEnabled(changable);
    restTremorRight.setEnabled(changable);
    posturalTremorLeft.setEnabled(changable);
    posturalTremorRight.setEnabled(changable);
    actionTremorLeft.setEnabled(changable);
    actionTremorRight.setEnabled(changable);
    openingLeft.setEnabled(changable);
    openingRight.setEnabled(changable);
    tappingLeft.setEnabled(changable);
    tappingRight.setEnabled(changable);
    newButton.setEnabled(changable);
    editButton.setEnabled(changable && !dataList.isSelectionEmpty());
    showButton.setEnabled(changable && !dataList.isSelectionEmpty());
    changeButton.setEnabled(!changable);
  }

  /**
   * called when panel is removed and changed
   */
  protected boolean remove() {
    this.saveData();
    return true;
  }

  /**
   * store visit to disk
   */
  private void saveData() {
    visit.setDiagnose(diagnose.getText());
    visit.setMemo(memo.getText());
    visit.setValid(valid.isSelected());
    visit.setVisitDate((java.util.Date)visitDate.getValue());
    visit.setTappingLeft(tappingLeft.getSelectedIndex());
    visit.setTappingRight(tappingRight.getSelectedIndex());
    visit.setOpeningLeft(openingLeft.getSelectedIndex());
    visit.setOpeningRight(openingRight.getSelectedIndex());
    visit.setRestTremorLeft(restTremorLeft.getSelectedIndex());
    visit.setRestTremorRight(restTremorRight.getSelectedIndex());
    visit.setPosturalTremorLeft(posturalTremorLeft.getSelectedIndex());
    visit.setPosturalTremorRight(posturalTremorRight.getSelectedIndex());
    visit.setActionTremorLeft(actionTremorLeft.getSelectedIndex());
    visit.setActionTremorRight(actionTremorRight.getSelectedIndex());
    visit.save();
  }

  protected void showSelectedData() {
    if(dataList.getSelectedValue() != null) {
      DataViewerFrame dvf = new DataViewerFrame(visit, (Data) dataList.getSelectedValue());
      Dimension dlgSize = dvf.getPreferredSize();
      Dimension frmSize = mf.getSize();
      Point loc = mf.getLocation();
      dvf.setLocation( (frmSize.width - dlgSize.width) / 2 + loc.x,
                      (frmSize.height - dlgSize.height) / 2 + loc.y);
      dvf.pack();
      dvf.setVisible(true);
    }
  }
}

/**
 * <p>Combo box with option for tremor</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class MyCombo extends JComboBox implements ActionListener {
  /**
   * parent VisitPanel
   */
  private VisitPanel adaptee;
  /**
   * dimesion of component
   */
  private Dimension dim = new Dimension(50, (int) this.getPreferredSize().getHeight());

  /**
   * create option box for VisitPanel and initial value
   */
  public MyCombo(VisitPanel adaptee, int value) {
    super();
    this.adaptee = adaptee;
    Object[] map = {"0", "1", "2", "3", "4"};
    for(Object i: map) {
      this.addItem(i);
    }
    this.setSelectedIndex(value);
    this.addActionListener(this);
  }

  /**
   * return minimal dimension
   */
  public Dimension getMinimumSize() {
    return dim;
  }

  /**
   * method is called when option is changed
   */
  public void actionPerformed(ActionEvent e) {
    adaptee.setChanged(true);
  }
}
