/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.loaders.XML;

import spirals.motol.database.*;
import spirals.motol.loaders.*;
import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;
import javax.swing.JOptionPane;
import java.util.zip.*;
import javax.swing.JFileChooser;

/**
 * <p>Tda, kter se star o nahrn databze z XML loit.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class XMLDatabaseLoader
    extends DatabaseLoader {
  private static final String databaseTag = "database";
  private static final String backupAttr = "backup";
  private static final String file = "database.xml";
  /**
   * defaultn adres s databz
   */
  public static File defaultRootFile = new File("data");
  /**
   * jmno souboru s databz
   */
  private File root = null;
  /**
   * cesta k souboru s databz
   */
  private File databaseFile = null;

  /**
   * tda pro parsovn XML souboru
   */
  public static final SAXBuilder builder = new SAXBuilder(
      "org.apache.xerces.parsers.SAXParser");
  /**
   * tda pro zpis XML souboru
   */
  public static final XMLOutputter serializer = new XMLOutputter();

  public XMLDatabaseLoader(File rootFile) {
    serializer.setFormat(org.jdom.output.Format.getPrettyFormat());
    this.root = rootFile;
    this.databaseFile = new File(this.root, this.file);
  }

  public File getRootFile() {
    return root;
  }

  public File getDatabaseFile() {
    return databaseFile;
  }

  /**
   * @return PacientLoader vrac XMLPacientLoader
   */
  public PacientLoader getDefaultPacientLoader() {
    return XMLPacientLoader.getInstance();
  }

  /**
   * @param database Database nahraje do tdy databzi
   */
  public void load(Database database) {
    if (databaseFile.exists()) {
      try {
        Document doc = builder.build(databaseFile);
        Element root = doc.getRootElement();
        database.setLastBackup(Engine.getSimpleDate(root.getAttributeValue(
            backupAttr)));
        List<Element> pacients = root.getChildren(XMLPacientLoader.pacientTag);
        database.setSize(pacients.size());
        for (Element e : pacients) {
//        database.addPacient(new Pacient(database, e.getAttributeValue(XMLPacientLoader.IDAttr)));
          database.addPacient( ( (XMLPacientLoader) XMLPacientLoader.
                                getInstance()).
                              load(database, e));
        }
      }
      catch (JDOMException e) {
        System.err.println("Chyba parsovani souboru databaze.");
        System.err.println(e.getMessage());
      }
      catch (IOException e) {
        System.err.println(e);
      }
    }
    else {
      System.err.println("Zakladam novou databazi");
      JOptionPane.showMessageDialog(database.getEngine().getMainFrame(),
                                    "Zakldm novou databzi",
                                    "Nov databze",
                                    JOptionPane.INFORMATION_MESSAGE);
      if (root.exists() || root.mkdir()) {
        this.save(database);
      }
      else {
        System.err.println("Nelze zalozit novou databazi");
        System.exit(1);
      }
    }
  }

  /**
   * ulo databzi do souboru
   * @param database Database
   */
  public void save(Database database) {
    Element root = new Element(databaseTag);
    root.setAttribute(backupAttr,
                      Engine.getSimpleDateFormat().format(database.
        getLastBackup()));
    for (Pacient p : database.getAll()) {
      root.addContent(XMLPacientLoader.getElement(p));
    }

    Document doc = new Document(root);
    FileOutputStream os = null;
    try {
      os = new FileOutputStream(databaseFile);
      serializer.output(doc, os);
    }
    catch (IOException e) {
      System.err.println(e);
    }
    finally {
      if (os != null) {
        try {
          os.close();
        }
        catch (IOException e) {}
      }
    }
  }

  /**
   * vrt velikost databze na disku
   * @param database Database
   * @return long
   */
  public long getDatabaseSize(Database database) {
    long size = 0;
    for (File file : this.root.listFiles()) {
      if (file.isDirectory()) {
        for (File f : file.listFiles()) {
          size += f.length();
        }
      }
      else {
        size += file.length();
      }
    }
    return size;
  }

  /**
   * zanonymizuje data a ulo je do zip souboru
   * @param database Database
   */
  public void exportData(Database database) {
    JFileChooser fc = new JFileChooser();
    fc.setSelectedFile(new File("export.zip"));
    if (fc.showSaveDialog(database.getEngine().getMainFrame()) ==
        JFileChooser.APPROVE_OPTION) {
      this.exportData(database, fc.getSelectedFile());
    }
  }

  private void exportData(Database database, File outFile) {
    ZipOutputStream out = null;
    try {
      out = new ZipOutputStream(new FileOutputStream(outFile));
      out.setMethod(ZipOutputStream.DEFLATED);
      out.setLevel(6);

      // create root element of database
      Element root = new Element(databaseTag);
      root.setAttribute(backupAttr,
                        Engine.getSimpleDateFormat().format(new Date()));
      long newID = 1;
      List<Pacient> listAll = database.getAll();
      Collections.shuffle(listAll);
      // export pacients
      for (Pacient p : listAll) {
        // name pacients in sequence begining 00000001
        String ID = stringID(newID);
        Element pacient = XMLPacientLoader.getElement(p.getAnonymus(ID));
        root.addContent(pacient);
        // for all visits of pacient
        for (Visit v : p.getVisits()) {
          String visitFile = XMLVisitLoader.getFile(v, ID, true).getParentFile().
              getName() + File.separator +
              XMLVisitLoader.getFile(v, ID, true).getName();
          out.putNextEntry(new ZipEntry(visitFile));
          copy(new FileInputStream(XMLVisitLoader.getFile(v, true)), out);
          // for all data in visit
          for (Data d : v.getData()) {
            String dataFile = XMLDataLoader.getFile(d, ID, true).getParentFile().
                getName() + File.separator +
                XMLDataLoader.getFile(d, ID, true).getName();
            out.putNextEntry(new ZipEntry(dataFile));
            copy(new FileInputStream(XMLDataLoader.getFile(d, true)), out);
          }
        }
        newID++;
      }

      // copy root file
      Document doc = new Document(root);
      out.putNextEntry(new ZipEntry(file));
      serializer.output(doc, out);
      System.err.println("Databaze exportovana");
      JOptionPane.showMessageDialog(database.getEngine().getMainFrame(),
                                    "Export databze probhl",
                                    "Export databze",
                                    JOptionPane.INFORMATION_MESSAGE);
    }
    catch (IOException e) {
      System.err.println("Nelze exportovat databazi");
    }
    finally {
      if (out != null) {
        try {
          out.close();
        }
        catch (Exception e) {
          System.err.println("Chyba pri zavirani souboru");
        }
      }
    }
  }

  /**
   * Vrt devtiznakovou reprezentaci sla. dopluje zleva nuly.
   * @param ID long
   * @return String
   */
  private static String stringID(long ID) {
    Formatter f = new Formatter();
    return f.format("%09d", ID).toString();
  }

  /**
   * Zazipuje adres
   * @param zipDir File jmno adrese k zazipovn
   * @param zos ZipOutputStream vstupn proud
   * @param saveDir File cesta od koene
   * @throws IOException
   */
  public static final void zipDir(File zipDir, ZipOutputStream zos,
                                  File saveDir) throws IOException {
    for (File f : zipDir.listFiles()) {
      if (f.isDirectory()) {
        zipDir(f, zos, new File(saveDir, f.getName()));
      }
      else {
        FileInputStream fis = new FileInputStream(f);
        ZipEntry anEntry = new ZipEntry(saveDir + File.separator + f.getName());
        zos.putNextEntry(anEntry);
        copy(fis, zos);
        fis.close();
      }
    }
  }

  /**
   * Zkopruje adres
   * @param copyDir File cesta ke koprovanmu adresi
   * @param saveDir File cesta k vstupnmu adresi
   * @throws IOException
   */
  public static final void copyDir(File copyDir, File saveDir) throws
      IOException {
    saveDir.mkdir();
    for (File f : copyDir.listFiles()) {
      if (f.isDirectory()) {
        copyDir(f, new File(saveDir, f.getName()));
      }
      else {
        FileInputStream fis = new FileInputStream(f);
        FileOutputStream fos = new FileOutputStream(new File(saveDir, f.getName()));
        copy(fis, fos);
        fis.close();
        fos.close();
      }
    }
  }

  /**
   * Zkopruje vechna data z jednoho proudu do druhho
   * @param in InputStream
   * @param out OutputStream
   * @throws IOException
   */
  public static final void copy(InputStream in, OutputStream out) throws
      IOException {
    byte[] buf = new byte[2156];
    int len;
    while ( (len = in.read(buf)) > 0) {
      out.write(buf, 0, len);
    }
  }

  /**
   * Pipoj k databzi
   * @param database Database
   */
  public void importData(Database database) {
    JFileChooser fc = new JFileChooser();
    if (fc.showOpenDialog(database.getEngine().getMainFrame()) ==
        JFileChooser.APPROVE_OPTION) {
      this.importData(database, fc.getSelectedFile());
    }
  }

  /**
   * Pipoj k databzi obsah adrese.
   * @param database Database k jak databzi pipojit
   * @param inFile File adres druh XML databze
   */
  public void importData(Database database, File inFile) {
    ZipInputStream in = null;
    try {
      in = new ZipInputStream(new FileInputStream(inFile));
      if (this.deflateZipToTemp(root, in)) {
        Database temp = new Database(this);
        this.load(temp);
        this.merge(database, temp);
      }
      this.deleteTemp(root);
    }
    catch (IOException e) {
      System.err.println("Nelze importovat databazi");
    }
    finally {
      if (in != null) {
        try {
          in.close();
        }
        catch (Exception e) {
          System.err.println("Chyba pri zavirani souboru");
        }
      }
    }
  }

  /**
   * vytvo doasn adres pro spojovn databzi
   * @param temp File cesta k doasnmu adresi
   * @param in ZipInputStream odkud natat databzi
   * @return boolean
   */
  private boolean deflateZipToTemp(File temp, ZipInputStream in) {
    if (!temp.mkdir()) {
      System.err.println("Import: docasny adresar " + temp.getAbsolutePath() +
                         " nelze vytvorit");
    }
    else {
      ZipEntry ze;
      try {
        ze = in.getNextEntry();
        while (ze != null) {
          File f = new File(temp, ze.getName());
          if (!f.getParentFile().exists()) {
            if (!f.getParentFile().mkdir()) {
              System.err.println("Import: adresar " +
                                 f.getParentFile().getAbsolutePath() +
                                 " nelze vytvorit");
              return false;
            }
          }
          try {
            FileOutputStream fos = new FileOutputStream(f);
            copy(in, fos);
            fos.close();
          }
          catch (IOException e) {
            System.err.println("Import: soubor " + f.getAbsolutePath() +
                               " nelze zkopirovat");
            return false;
          }
          try {
            ze = in.getNextEntry();
          }
          catch (IOException e) {
            System.err.println("Import: vstupni soubor je poskozen");
            return false;
          }
        }
      }
      catch (IOException e) {
        return false;
      }
    }
    return true;
  }

  /**
   * smae doasn adres po spojen databz
   * @param temp File
   */
  private void deleteTemp(File temp) {
    if (temp.isDirectory()) {
      for (File f : temp.listFiles()) {
        deleteTemp(f);
      }
      if (!temp.delete()) {
        System.out.println("Import: docasny adresar " + temp.getAbsolutePath() +
                           " se nepodarilo smazat");
      }
    }
    else if (!temp.delete()) {
      System.out.println("Import: docasny soubor " + temp.getAbsolutePath() +
                         " se nepodarilo smazat");
    }
  }
}
