/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.loaders.XML;

import spirals.motol.loaders.*;
import spirals.motol.database.*;
import java.util.*;
import java.text.*;
import java.io.*;
import org.jdom.*;

/**
 * <p>Tda, kter se star o nahrn pacienta z XML loit.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class XMLPacientLoader
    extends PacientLoader {
  public static final String pacientTag = "pacient";
  public static final String IDAttr = "ID";
  public static final String firstnameAttr = "firstname";
  public static final String surnameAttr = "surname";
  public static final String bornAttr = "born";
  public static final String rhpAttr = "rhp";

  protected XMLPacientLoader() {}

  public static PacientLoader getInstance() {
    if (instance == null) {
      instance = new XMLPacientLoader();
    }
    return instance;
  }

  public VisitLoader getDefaultVisitLoader() {
    return XMLVisitLoader.getInstance();
  }

  public void load(Pacient pacient, String ID) {
    try {
      Document doc = XMLDatabaseLoader.builder.build( ( (XMLDatabaseLoader)
          pacient.getDatabase().getLoader()).getDatabaseFile());
      Element root = doc.getRootElement();
      Iterator i = root.getDescendants();
      while (i.hasNext()) {
        Object o = i.next();
        if (o instanceof Element) {
          Element e = (Element) o;
          String eId = e.getAttributeValue(IDAttr);
          if ( (eId != null) && eId.equals(ID)) {
            this.setPacient(pacient, e);
            break;
          }
        }
      }
    }
    catch (JDOMException e) {
      System.err.println("Chyba parsovani souboru databaze.");
      System.err.println(e.getMessage());
    }
    catch (IOException e) {
      System.err.println(e);
    }
  }

  /**
   * Nahraje do databze pacienta z XML elementu
   * @param database Database
   * @param el Element
   * @return Pacient
   */
  public Pacient load(Database database, Element el) {
    Pacient p = new Pacient(database);
    this.setPacient(p, el);
    return p;
  }

  /**
   * Zape obsah XML elementu do pacienta
   * @param p Pacient
   * @param e Element
   */
  private void setPacient(Pacient p, Element e) {
    p.setBornDate(Engine.getSimpleDate(e.getAttributeValue(bornAttr)));
    p.setFirstname(e.getAttributeValue(firstnameAttr));
    p.setSurname(e.getAttributeValue(surnameAttr));
    p.setRightHanded(e.getAttributeValue(rhpAttr).equals("1") ? true : false);
    p.setID(e.getAttributeValue(IDAttr));
  }

  public void save(Pacient pacient) {
    pacient.getDatabase().getLoader().save(pacient.getDatabase());
  }

  /**
   * vytvo z pacienta XML element
   * @param p Pacient
   * @return Element
   */
  public static Element getElement(Pacient p) {
    Element e = new Element(pacientTag);
    e.setAttribute(IDAttr, p.getID());
    e.setAttribute(firstnameAttr, p.getFirstname());
    e.setAttribute(surnameAttr, p.getSurname());
    e.setAttribute(rhpAttr, p.isRightHanded() ? "1" : "0");
    e.setAttribute(bornAttr, Engine.getSimpleDateFormat().format(p.getBornDate()));
    return e;
  }

  public List<Visit> getVisits(Pacient pacient) {
    List<Visit> visits = new ArrayList<Visit> (0);
    File file = new File( ( (XMLDatabaseLoader) pacient.getDatabase().getLoader()).
                         getRootFile(), pacient.getID());
    File[] list = file.listFiles(new FileFilter() {
      public boolean accept(File file) {
        String name = file.getName();
        int index = name.lastIndexOf(".");
        if ( (index > 0) && name.substring(index).equals(".xml")) {
          return true;
        }
        return false;
      }
    });
    if ( (list != null) && (pacient.getID().length() > 0)) {
      // load this files
      visits = new ArrayList(list.length);
      for (File f : list) {
        Date date = null;
        DateFormat df = new SimpleDateFormat(XMLVisitLoader.fileFormat +
                                             "'.xml'");
        try {
          date = df.parse(f.getName());
          visits.add(new Visit(pacient, date));
        }
        catch (ParseException e) {
          System.err.println("Chyba parsovani dat");
        }
      }
    }
    return visits;
  }

  public boolean replace(Pacient pacient, String oldID, String newID) {
    File dir = new File( ( (XMLDatabaseLoader) pacient.getDatabase().getLoader()).
                        getRootFile(), oldID);
    if (dir.exists() &&
        !dir.renameTo(new File( ( (XMLDatabaseLoader) pacient.getDatabase().
                                 getLoader()).getRootFile(), newID))) {
      System.err.println("Chyba zmeny ID pacienta");
      return false;
    }
    return true;
  }
}
