/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.loaders.XML;

import spirals.motol.loaders.*;
import spirals.motol.database.*;
import java.util.*;
import java.text.*;
import java.io.*;
import org.jdom.*;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

/**
 * <p>Tda, kter se star o nahrn nvtvy z XML loit.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class XMLVisitLoader
    extends VisitLoader {
  public static final String fileFormat = "yyyy_MM_dd_HH_mm";
  public static final String visitTag = "visit";
  public static final String memoTag = "memo";
  public static final String diagnoseTag = "diagnose";
  public static final String validAttr = "valid";
  public static final String restLeftAttr = "restLeft";
  public static final String restRightAttr = "restRight";
  public static final String postLeftAttr = "postLeft";
  public static final String postRightAttr = "postRight";
  public static final String actionLeftAttr = "actionLeft";
  public static final String actionRightAttr = "actionRight";
  public static final String openLeftAttr = "openLeft";
  public static final String openRightAttr = "openRight";
  public static final String tapLeftAttr = "tapLeft";
  public static final String tapRightAttr = "tapRight";

  public static final SAXBuilder builder = new SAXBuilder(
      "org.apache.xerces.parsers.SAXParser");
  public static final XMLOutputter serializer = new XMLOutputter();

  private XMLVisitLoader() {
    serializer.setFormat(org.jdom.output.Format.getPrettyFormat());
  }

  public static VisitLoader getInstance() {
    if (instance == null) {
      instance = new XMLVisitLoader();
    }
    return instance;
  }

  public DataLoader getDefaultDataLoader() {
    return XMLDataLoader.getInstance();
  }

  public void load(Visit visit, Pacient pacient, Date date) {
    try {
      Document doc = builder.build(getFile(visit, true));
      this.setVisit(visit, doc.getRootElement());
    }
    catch (JDOMException e) {
      System.err.println("Chyba parsovani souboru databaze.");
      System.err.println(e.getMessage());
    }
    catch (IOException e) {
      System.err.println(e);
    }
  }

  /**
   * Zape obsah XML elementu do nvtvy
   * @param v Visit
   * @param e Element
   */
  private void setVisit(Visit v, Element e) {
    v.setValid(Integer.parseInt(e.getAttributeValue(validAttr)) == 1 ? true : false);
    v.setRestTremorLeft(Integer.parseInt(e.getAttributeValue(restLeftAttr)));
    v.setRestTremorRight(Integer.parseInt(e.getAttributeValue(restRightAttr)));
    v.setPosturalTremorLeft(Integer.parseInt(e.getAttributeValue(postLeftAttr)));
    v.setPosturalTremorRight(Integer.parseInt(e.getAttributeValue(postRightAttr)));
    v.setActionTremorLeft(Integer.parseInt(e.getAttributeValue(actionLeftAttr)));
    v.setActionTremorRight(Integer.parseInt(e.getAttributeValue(actionRightAttr)));
    v.setOpeningLeft(Integer.parseInt(e.getAttributeValue(openLeftAttr)));
    v.setOpeningRight(Integer.parseInt(e.getAttributeValue(openRightAttr)));
    v.setTappingLeft(Integer.parseInt(e.getAttributeValue(tapLeftAttr)));
    v.setTappingRight(Integer.parseInt(e.getAttributeValue(tapRightAttr)));
    v.setMemo(e.getChild(memoTag).getText());
    v.setDiagnose(e.getChild(diagnoseTag).getText());

    // load data
    List<Element> list = e.getChildren(XMLDataLoader.dataTag);
    for (Element el : list) {
      /*DateFormat df = new SimpleDateFormat(XMLDataLoader.dateFormat + "'.zip'");
             try {
       v.addData(new Data(v, df.parse(el.getAttributeValue(XMLDataLoader.fileAttr))));
            } catch(ParseException ex) {
              System.err.println("Chyba parsovani dat");
            }
       */
      v.addData( ( (XMLDataLoader) XMLDataLoader.getInstance()).load(v, el));
    }
  }

  public void save(Visit visit) {
    File dir = new File( getRootFile(visit, false), visit.getPacient().getID());
    if (!dir.exists()) {
      dir.mkdir();
    }

    Document doc = new Document(getElement(visit));

    OutputStream os = null;
    try {
      os = new FileOutputStream(getFile(visit, false));
      serializer.output(doc, os);
    }
    catch (IOException e) {
      System.err.println(e);
    }
    finally {
      if (os != null) {
        try {
          os.close();
        }
        catch (IOException e) {}
      }
    }
  }

  /**
   * vytvo z nvtvy XML element
   * @param v Visit
   * @return Element
   */
  public static Element getElement(Visit v) {
    Element root = new Element(visitTag);
    root.setAttribute(validAttr, v.isValid() ? "1" : "0");
    root.setAttribute(restLeftAttr, Integer.toString(v.getRestTremorLeft()));
    root.setAttribute(restRightAttr, Integer.toString(v.getRestTremorRight()));
    root.setAttribute(postLeftAttr, Integer.toString(v.getPosturalTremorLeft()));
    root.setAttribute(postRightAttr, Integer.toString(v.getPosturalTremorRight()));
    root.setAttribute(actionLeftAttr, Integer.toString(v.getActionTremorLeft()));
    root.setAttribute(actionRightAttr, Integer.toString(v.getActionTremorRight()));
    root.setAttribute(openLeftAttr, Integer.toString(v.getOpeningLeft()));
    root.setAttribute(openRightAttr, Integer.toString(v.getOpeningRight()));
    root.setAttribute(tapLeftAttr, Integer.toString(v.getTappingLeft()));
    root.setAttribute(tapRightAttr, Integer.toString(v.getTappingRight()));
    Element diag = new Element(diagnoseTag);
    diag.setText(v.getDiagnose());
    root.addContent(diag);
    Element m = new Element(memoTag);
    m.setText(v.getMemo());
    root.addContent(m);
    for (Data d : v.getData()) {
      root.addContent(XMLDataLoader.getElement(d));
    }
    return root;

  }

  /**
   * Vytvo cestu pro nvtvu a pacienta
   * @param visit Visit
   * @param ID String ID pacienta, ke ktermu nvtva pat / maj patit
   * @param load boolean true, pokud nahrvm nvtvu z temp databze pi sluovn
   * @return File cesta k souboru s nvtvou
   */
  public static File getFile(Visit visit, String ID, boolean load) {
    DateFormat df = new SimpleDateFormat(fileFormat);
    return new File(new File(getRootFile(visit, load), ID),
                    df.format(visit.getVisitDate()) + ".xml");
  }

  /**
   * Vytvo cestu pro nvtvu
   * @param visit Visit
   * @param load boolean true, pokud nahrvm nvtvu z temp databze pi sluovn
   * @return File cesta k souboru s nvtvou
   */
  public static File getFile(Visit visit, boolean load) {
    return getFile(visit, visit.getPacient().getID(), load);
  }

  /**
   * zjisti adres ve kterm je soubor s nvtvou uloen
   * @param visit Visit
   * @param load boolean true, pokud nahrvm nvtvu z temp databze pi sluovn
   * @return File
   */
  public static File getRootFile(Visit visit, boolean load) {
    DatabaseLoader dl = visit.getPacient().getDatabase().getLoader();
    if (load && (dl instanceof XMLDatabaseLoader)) {
      return ( (XMLDatabaseLoader) dl).getRootFile();
    }
    else {
      return XMLDatabaseLoader.defaultRootFile;
    }
  }
}
